Package: polish
Title: Polishing Content for 'Word' and 'PowerPoint'
Version: 0.3.1
Authors@R: c(
      person("Romain", "François", email = "romain@tada.science", role = "aut"), 
      person("Ellis", "Hughes", email = "ellis.h.hughes@gsk.com", role = c("aut", "cre")), 
      person("Shannon", "Haughton", email = "shannon.l.haughton@gsk.com", role = "aut"),
      person("GlaxoSmithKline Research & Development Limited", role = c("cph", "fnd"))
    )
Description: Set of functions to polish content for Microsoft 'Word' and 'PowerPoint' 
    into 'OOXML'. Polishing is the conversion of the R object into an 'OOXML'
    representation of the object that can then be added to 'Word' or 'PowerPoint' 
    files.
License: Apache License (>= 2)
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://gsk-biostatistics.github.io/polish/,
        https://github.com/GSK-Biostatistics/polish/
BugReports: https://github.com/GSK-Biostatistics/polish/issues/
Depends: R (>= 4.1.0)
Imports: cli, commonmark, dplyr, flextable, ggplot2, glue, gt,
        htmltools, magick, officer (>= 0.6.9), purrr, rlang, rmarkdown,
        sloop, tibble, webshot2, xml2
Suggests: testthat (>= 3.0.0), withr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-30 17:40:34 UTC; ehh82309
Author: Romain François [aut],
  Ellis Hughes [aut, cre],
  Shannon Haughton [aut],
  GlaxoSmithKline Research & Development Limited [cph, fnd]
Maintainer: Ellis Hughes <ellis.h.hughes@gsk.com>
Repository: CRAN
Date/Publication: 2026-06-03 14:10:12 UTC
Built: R 4.5.2; ; 2026-06-03 17:37:07 UTC; unix

Package: polish
Title: Polishing Content for 'Word' and 'PowerPoint'
Version: 0.3.1
Authors@R: c(
      person("Romain", "François", email = "romain@tada.science", role = "aut"), 
      person("Ellis", "Hughes", email = "ellis.h.hughes@gsk.com", role = c("aut", "cre")), 
      person("Shannon", "Haughton", email = "shannon.l.haughton@gsk.com", role = "aut"),
      person("GlaxoSmithKline Research & Development Limited", role = c("cph", "fnd"))
    )
Description: Set of functions to polish content for Microsoft 'Word' and 'PowerPoint' 
    into 'OOXML'. Polishing is the conversion of the R object into an 'OOXML'
    representation of the object that can then be added to 'Word' or 'PowerPoint' 
    files.
License: Apache License (>= 2)
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://gsk-biostatistics.github.io/polish/,
        https://github.com/GSK-Biostatistics/polish/
BugReports: https://github.com/GSK-Biostatistics/polish/issues/
Depends: R (>= 4.1.0)
Imports: cli, commonmark, dplyr, flextable, ggplot2, glue, gt,
        htmltools, magick, officer (>= 0.6.9), purrr, rlang, rmarkdown,
        sloop, tibble, webshot2, xml2
Suggests: testthat (>= 3.0.0), withr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-30 17:40:34 UTC; ehh82309
Author: Romain François [aut],
  Ellis Hughes [aut, cre],
  Shannon Haughton [aut],
  GlaxoSmithKline Research & Development Limited [cph, fnd]
Maintainer: Ellis Hughes <ellis.h.hughes@gsk.com>
Repository: CRAN
Date/Publication: 2026-06-03 14:10:12 UTC
Built: R 4.5.2; ; 2026-06-03 16:08:35 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as_xml_nodeset,character)
S3method(as_xml_nodeset,glue)
S3method(as_xml_nodeset,list)
S3method(as_xml_nodeset,xml_node)
S3method(as_xml_nodeset,xml_nodeset)
S3method(polish_content_pptx,as_is)
S3method(polish_content_pptx,as_md)
S3method(polish_content_pptx,character)
S3method(polish_content_pptx,data.frame)
S3method(polish_content_pptx,default)
S3method(polish_content_pptx,file)
S3method(polish_content_pptx,file_html)
S3method(polish_content_pptx,file_jpeg)
S3method(polish_content_pptx,file_md)
S3method(polish_content_pptx,file_pdf)
S3method(polish_content_pptx,file_png)
S3method(polish_content_pptx,file_rtf)
S3method(polish_content_pptx,file_svg)
S3method(polish_content_pptx,file_txt)
S3method(polish_content_pptx,flextable)
S3method(polish_content_pptx,ggplot)
S3method(polish_content_pptx,list)
S3method(polish_content_pptx,numeric)
S3method(polish_content_word,as_is)
S3method(polish_content_word,as_md)
S3method(polish_content_word,character)
S3method(polish_content_word,data.frame)
S3method(polish_content_word,default)
S3method(polish_content_word,file)
S3method(polish_content_word,file_docx)
S3method(polish_content_word,file_html)
S3method(polish_content_word,file_jpeg)
S3method(polish_content_word,file_md)
S3method(polish_content_word,file_png)
S3method(polish_content_word,file_rtf)
S3method(polish_content_word,file_svg)
S3method(polish_content_word,file_txt)
S3method(polish_content_word,flextable)
S3method(polish_content_word,ggplot)
S3method(polish_content_word,gt_group)
S3method(polish_content_word,gt_tbl)
S3method(polish_content_word,numeric)
S3method(print,file)
S3method(print,polish_xml_nodeset)
export(as_file)
export(as_is)
export(as_md)
export(as_xml_node)
export(as_xml_node_pptx)
export(as_xml_node_word)
export(as_xml_nodeset)
export(as_xml_pptx)
export(as_xml_word)
export(polish_content)
export(polish_content_pptx)
export(polish_content_word)
export(show_xml)
export(sp_shell)
export(xml_nodeset_ns_spec)
import(cli)
import(glue)
import(htmltools)
import(rlang)
importFrom(commonmark,markdown_xml)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,na_if)
importFrom(dplyr,slice)
importFrom(flextable,dim_pretty)
importFrom(flextable,flextable)
importFrom(flextable,height)
importFrom(flextable,height_all)
importFrom(flextable,nrow_part)
importFrom(flextable,width)
importFrom(ggplot2,ggsave)
importFrom(grDevices,col2rgb)
importFrom(grDevices,rgb)
importFrom(gt,as_word)
importFrom(gt,grp_pull)
importFrom(gt,vec_fmt_markdown)
importFrom(purrr,imap)
importFrom(purrr,map)
importFrom(sloop,s3_dispatch)
importFrom(tibble,tibble)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,getFromNamespace)
importFrom(xml2,"xml_attr<-")
importFrom(xml2,read_xml)
importFrom(xml2,xml_add_child)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_attrs)
importFrom(xml2,xml_child)
importFrom(xml2,xml_children)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_find_first)
importFrom(xml2,xml_name)
importFrom(xml2,xml_ns)
importFrom(xml2,xml_remove)
importFrom(xml2,xml_replace)
importFrom(xml2,xml_text)

# polish  <a href="https://github.com/GSK-Biostatistics/polish/"><img src="man/figures/logo.png" align="right" height="138" /></a>

<!-- badges: start -->
[![R-CMD-check](https://github.com/GSK-Biostatistics/polish/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/GSK-Biostatistics/polish/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

The goal of polish is to convert R objects into ooxml that is viable for Word documents and PowerPoint presentations.

## Installation

You can install the development version of polish from [GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("GSK-Biostatistics/polish")
```

## Example

This is a basic example which shows you how to convert R objects into ooxml:

``` r
library(polish)
## basic example code

## Make word based ooxml
polish_content("{polish} is a great package!", type = "word")

## Make pptx based ooxml
polish_content("{polish} is a great package!", type = "pptx")

```

#' @import cli
#' @import glue
#' @import rlang
#' @import htmltools
#' @importFrom xml2 read_xml xml_children xml_attrs xml_find_first xml_name xml_child xml_attr xml_attr<- xml_ns xml_find_all xml_replace xml_text
#' @importFrom tools file_ext file_path_sans_ext
#' @importFrom purrr map imap
#' @importFrom ggplot2 ggsave
#' @importFrom gt vec_fmt_markdown as_word grp_pull
#' @importFrom utils getFromNamespace
#' @importFrom flextable flextable
#' @importFrom sloop s3_dispatch
#' @importFrom utils capture.output browseURL
NULL

ns_flextable <- NULL

.onLoad <- function(libname, pkgname) {
  ns_flextable <<- asNamespace("flextable")
}

# as_file() ---------------------------------------------------------------

#' Mark as a file by adding classes
#'
#' @param path file path
#'
#' @return `path` with classes set to `"file"` and `"file_{ext}"` where `ext` is
#'         the file extension
#'
#' @examples
#' tf <- tempfile(fileext = ".txt")
#' writeLines("", tf)
#'
#' as_file(tf)
#'
#' @export
as_file <- function(path) {
  check_file_exists(path)
  ext <- paste0("file_", file_extension(path))

  class(path) <- c(ext, "file")
  path
}

#' @export
print.file <- function(x, ...) {
  cli_text("File {.file {x}}")
}

check_file_exists <- function(path, error_call = caller_env()) {
  if (!file.exists(path)) {
    cli_abort("File {.file {path}} does not exist.", call = error_call)
  }
}

file_extension <- function(path, error_call = caller_env()) {
  ext <- file_ext(path)
  if (identical(ext, "")) {
    cli_abort("File {.file {path}} has no extension.", call = error_call)
  }
  tolower(ext)
}


ensure_dir_exists <- function(dir) {
  if (!dir.exists(dir)){
    dir.create(dir)
  }
}

change_extension <- function(path, extension) {
  prefix <- file_path_sans_ext(basename(path))
  glue("{prefix}.{extension}")
}

test_that("as_file() errors ", {
  expect_error(as_file(tempfile()))
  expect_error(local_file())
})

test_that("as_file() set extension specific classes", {
  expect_s3_class(local_file(".txt") , c("file_txt", "file"))
  expect_s3_class(local_file(".docx"), c("file_docx", "file"))
  expect_s3_class(local_file(".pptx"), c("file_pptx", "file"))
})

test_that("polish_content_word(<file_txt>)", {
  tf <- withr::local_tempfile(fileext = ".txt")
  writeLines(c("hello", "world"), con = tf)

  expect_snapshot(
    polish_content_word(as_file(tf), inline = TRUE)
  )

  expect_snapshot(
    polish_content_word(as_file(tf), inline = FALSE)
  )
})

test_that("polish_content_word(<file_md>)", {
  tf <- withr::local_tempfile(fileext = ".md")
  writeLines(c("hello", "*world*"), con = tf)

  expect_snapshot(
    polish_content_word(as_file(tf))
  )
})

test_that("polish_content_word(<file_docx>", {
  suppressWarnings(withr::local_package("officer")) # for print(), read_docx()

  tf <- withr::local_tempfile(fileext = ".docx")
  doc <- read_docx()
  print(doc, tf)

  expect_snapshot(transform = transform_polish,
    polish_content_word(as_file(tf))
  )
})

test_that("polish_content_word(<file_rtf>", {
  tf <- withr::local_tempfile(fileext = ".rtf")
  writeLines(c("hello", "rtf"), tf)

  expect_snapshot(transform = transform_polish,
    polish_content_word(as_file(tf))
  )
})

test_that("polish_content_word(<file_html>)", {

  tf <- withr::local_tempfile(fileext = ".html")
  writeLines('<html><body>hello</body></html>', tf)

  expect_snapshot(transform = transform_polish,
                  polish_content_word(as_file(tf))
  )
})

test_that("polish_content_pptx(<file_txt>)", {
  tf <- withr::local_tempfile(fileext = ".txt")
  writeLines(c("hello", "world"), con = tf)

  expect_snapshot(error = TRUE,
    polish_content_pptx(as_file(tf))
  )
})

test_that("polish_content_pptx(<file_md>)", {
  tf <- withr::local_tempfile(fileext = ".md")
  writeLines(c("hello", "*world*"), con = tf)

  expect_snapshot(
    polish_content_pptx(as_file(tf))
  )

})

test_that("polish_content_pptx(<file_rtf>)", {
  skip_on_ci()
  skip_on_cran()

  tf <- withr::local_tempfile(fileext = ".rtf")
  writeLines(c("hello", "rtf"), tf)

  expect_snapshot(transform = transform_polish,
    polish_content_pptx(as_file(tf))
  )

})

test_that("polish_content_pptx(<file_html>)", {
  skip_on_ci()
  skip_on_cran()

  tf <- withr::local_tempfile(fileext = ".html")
  writeLines('<html><body>hello</body></html>', tf)

  expect_snapshot(transform = transform_polish,
    polish_content_pptx(as_file(tf))
  )
})

not_yet_implemented <- function(what, issue, error_call = caller_env()) {
  url <- glue("https://github.com/GSK-Biostatistics/polish/issues/{issue}")
  cli_abort(call = error_call, c(
    "{what} is not yet implemented.",
    i = "See {.url {url}} for progress."
  ))
}

#' Show the xml of the polished version of a display in the browser
#'
#' @param xml an xml node or nodeset
#' @param wrap if TRUE, the xml is wrapped in a w:body node
#'
#' @details This opens the xml in the browser, so that you can do View source
#' and inspect the structure of the xml
#'
#' @returns used for side effect of previewing xml in a browser. Helpful for
#'   inspecting the OOXML generated by this package.
#'
#' @export
show_xml <- function(xml, wrap = TRUE) {
  file <- tempfile(fileext = ".html")
  con <- file(file, open = "w")

  if (isTRUE(wrap)) {
    writeLines('<w:body xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main" xmlns:wp="http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:w14="http://schemas.microsoft.com/office/word/2010/wordml">', con = con)
  }
  writeLines(as.character(xml), con = con)
  if (isTRUE(wrap)) {
    writeLines('</w:body>', con = con)
  }
  close(con)

  browseURL(file)
}

#' Check ... empty
#'
#' Wrapper around [rlang::check_dots_empty()] that only does the check
#' if the `polish.dots_error` is set to TRUE
#'
#' @inheritParams rlang::check_dots_empty
polish_check_dots_empty <- function(env = caller_env(), error = NULL, call = caller_env()) {
  if (isTRUE(getOption("polish.dots_error"))) {
    check_dots_empty(env = env, error = error, call = call)
  }
}

# word --------------------------------------------------------------------

#' @param align alignment of table and caption
#' @param split should table be allowed to split
#' @param topcaption should caption be above or below the table
#' @param keepnext should content be marked for keep next.
#' @rdname polish_content_word
#'
#' @export
polish_content_word.flextable <- function(x, inline = TRUE, ..., align = NULL, split = NULL, topcaption = TRUE, keepnext = NULL, error_call = current_env()) {
  polish_check_dots_empty(call = error_call)
  if (!is.null(align)) {
    x$properties$align <- align
  }
  if (!is.null(split)) {
    x$properties$opts_word$split <- split
  }
  if (!is.null(keepnext)) {
    x$properties$opts_word$keep_with_next <- keepnext
  }

  caption_xml <- NULL
  if (!is.null(x[["caption"]][["value"]])) {
    caption_xml <- ns_flextable$caption_default_docx_openxml(
      x = x,
      keep_with_next = topcaption,
      allow_autonum = TRUE
    )
  }

  tbl_xml <- officer::to_wml(x, add_ns = TRUE)

  xml_content <- paste0(
    if(topcaption & !is.null(caption_xml)){caption_xml},
    tbl_xml,
    if(!topcaption & !is.null(caption_xml)){caption_xml}
  )

  as_xml_word(xml_content, error_call = error_call)
}

# pptx --------------------------------------------------------------------

#' @param full_width Logical, used to fit full size of given width
#' @param page_size Number of rows to accommodate in a slide
#' @param table_style Named list with two entries: style and options. style is
#'   the name of table style to apply as a character, options a character vector
#'   of options to apply from the table style, including: "Banded Columns",
#'   "Banded Rows", "First Column", "Header Row", "Last Column", and "Total Row"
#' @param pptx a `pptx_container` object
#'
#' @rdname polish_content_pptx
#'
#' @export
polish_content_pptx.flextable <- function(x, ph = "<p:ph/>", ...,  full_width = FALSE, page_size = NULL, table_style = NULL, pptx = NULL, error_call = current_env()){

  polish_check_dots_empty(call = error_call)
  sp_ph <- sp_shell(ph)

  if (length(xml_find_all(sp_ph, ".//a:xfrm")) == 1) {
    offsets <- dim_extract(sp_ph, ".//a:off", c("x", "y"))
    dims    <- dim_extract(sp_ph, ".//a:ext", c("cx", "cy"))

    width   <- dims$cx
    height  <- dims$cy
    left    <- offsets$x
    top     <- offsets$y
  } else {
    width   <- 1
    height  <- 2
    left    <- 0
    top     <- 0
  }

  x <- autofit_ft(
    x,
    width = width, height = height,
    full_width = full_width, page_size = page_size
  )

  dim <- flextable::flextable_dim(x)

  offset_x <- left
  if (dim$widths > width) {
    cli_inform(c(
      "Table width is greater than the provided placeholder and may run off the slide.",
      i = "Consider splitting the table across multiple slides."
    ), error_call = error_call)
  } else {
    offset_x <- offset_x + (width - dim$widths) / 2
  }

  offset_y <- top
  if (dim$heights > height) {
    cli_inform(c(
      "Table height is greater than the provided placeholder and may run off the slide.",
      i = "Consider splitting the table across multiple slides."
    ), error_call = error_call)
  } else {
    ## center table in placeholder if not taller
    offset_y <- offset_y + (height - dim$heights) / 2
  }

  xml_content <- as_xml_node(ns_flextable$gen_raw_pml(
    x,
    offx = offset_x, offy = offset_y, cx = dim$widths, cy = dim$heights
  ))

  ## Add table style if it exists
  if(!is.null(table_style)){
    add_table_style(xml_content, table_style, pptx)
  }

  xml2::xml_replace(
    xml_find_all(xml_content, ".//p:cNvPr"),
    xml_find_all(as_xml_node(sp_ph), ".//p:cNvPr")
  )

  as_xml_pptx(xml_content, error_call = error_call)
}

#' Resize width and height of a flextable
#'
#' @param x a flextable
#' @param width maximum acceptance width
#' @param height maximum acceptance height
#' @param full_width Logical, used to fit full size of given width
#' @param page_size Number of rows to accommodate in a slide
#'
#' @importFrom flextable dim_pretty width height nrow_part height_all
#' @importFrom dplyr na_if
#'
#' @return Flextable
#'
#' @noRd
autofit_ft <- function(x, width, height, full_width = FALSE, page_size = NULL){
  dist_width <- dim(x)$widths
  x <- flextable::autofit(x)
  w <- flextable::dim_pretty(x)$widths
  h <- flextable::dim_pretty(x)$heights

  ## if any h values are 0, set to the average of non "0" h values
  if(sum(h <= 0, na.rm = TRUE) > 0){
    h[h<=0] <- mean(h[h>0], na.rm = TRUE)
  }

  # [flextable::autofit()] doesn't work properly if you have a spanning header.
  # So, get the width of header apart and the body part separately and compare
  # if any column width is more than expected as defined by `> 1` and replace
  # the width by standard width through [flextable::dim].
  if (nrow_part(x, "header") > 1) {
    w_body <- flextable::dim_pretty(x, part = "body")$widths

    # algorithm is based on trial and error method
    # - ratio is more than 100% then set constant width
    reconsider <- which(floor(pmax(w_body, w)/pmin(w_body, w)) > 1)

    new_w <- purrr::map2_dbl(dist_width, w_body, ~ sum(.x + .y)/2)
    w[reconsider] <-  new_w[reconsider]
  }

  x <- width(x, j = seq_along(w), w)

  if (full_width || sum(w) > width) {
    # fix if width is larger than layout size or req. to fit full width
    w_val <- (w / sum(w)) * width
    x <- width(x, width = w_val)
  } else if (sum(w) < width / 2) {
    # fix if width is smaller than half the size of layout
    w_val <- (w / sum(w)) * width / 2
    x <- width(x, width = w_val)
  }

  # height ---
  h_nrow <- nrow_part(x, "header")
  b_nrow <- nrow_part(x, "body")
  f_nrow <- nrow_part(x, "footer")

  cum_height <- ((h / sum(h)) * height - (0.1925525 / (h_nrow + b_nrow + f_nrow))) |>
    pmax(.2)

  # smallest height adjustment to fit
  body_h_val <- min(c(cum_height[(h_nrow+1):(h_nrow+b_nrow)],h[(h_nrow+1):(h_nrow+b_nrow)]),na.rm = TRUE )
  if(isTRUE(is.infinite(body_h_val))){
    body_h_val <- .37
  }
  x <- flextable::height_all(x, body_h_val, part = "body")

  if(f_nrow > 0){
    footer_h_val <- min(c(cum_height[(h_nrow+b_nrow+1):(h_nrow+b_nrow+f_nrow)],h[(h_nrow+b_nrow+1):(h_nrow+b_nrow+f_nrow)]), na.rm = TRUE)
    if(isTRUE(is.infinite(footer_h_val))){
      footer_h_val <- .37
    }
    x <- flextable::height_all(x, footer_h_val, part = "footer")
  }

  x
}

#' @importFrom xml2 xml_add_child xml_remove xml_find_all
add_table_style <- function(tbl_xml , table_style, pptx = NULL, error_call = current_env()){

  tblPr_node <- xml_find_first(tbl_xml, ".//a:tblPr")

    ## add table specific formatting
  tblPr_node <- add_table_style_settings(tblPr_node, table_style$options)

  if(!is.null(pptx)){

    check_table_style(table_style, error_call = error_call)

    ## add which style to use
    table_style_id_node <- get_table_style_id_node(table_style$style, pptx, error_call = error_call)
    if(!is.null(table_style_id_node)){

      tblPr_node |>
        xml_add_child(table_style_id_node)

      ## remove cell styles to let style override
      tbl_xml |>
        xml_find_all(".//a:tc//a:tcPr") |>
        xml_remove()

      ## remove celltext fills to let style override
      tbl_xml |>
        xml_find_all(".//a:tc//a:rPr//a:solidFill") |>
        xml_remove()

    }
  }

}

check_table_style <- function(x, error_call = current_env()){

  ## Nothing to check if NULL
  if(is.null(x)){
    return()
  }

  not_list <- !is.list(x)
  missing_style <- !"style" %in% names(x)
  missing_options <- !"options" %in% names(x)
  extra_names <- sum(!names(x) %in% c("style","options")) > 0
  extra_options <- FALSE

  if(not_list | (missing_style & missing_options) | extra_names | extra_options){
    cli_abort(c(
      "Argument `table_style` must be a named list with only two(2) named elements for `style` and `options`.",
      i = "`style` must match one of the existing Table Styles in the PowerPoint",
      i = paste0("`options` is a character vector of all the options: ",paste0("`",table_style_settings_key,"`", collapse = ", "))
    ), call = error_call)
  }

}

table_style_settings_key <- c(
  "bandCol" = "Banded Columns",
  "bandRow" = "Banded Rows",
  "firstCol" = "First Column",
  "firstRow"  = "Header Row",
  "lastCol" = "Last Column",
  "lastRow" = "Total Row"
)

add_table_style_settings <- function(tblPr_node, table_styles){

  styles <- names(table_style_settings_key)[table_style_settings_key %in% table_styles]

  for(style in styles){
    xml_attr(tblPr_node,style) <- 1
  }

  tblPr_node

}

#' @importFrom dplyr filter slice
get_table_style_id_node <- function(table_style_name, pptx, error_call = current_env()){

  if(is.null(table_style_name)){
    return(NULL)
  }

  table_style_df <- get_table_styles(pptx)

  style_id_df <- table_style_df |>
    filter(.data$style_name == table_style_name) |>
    slice(1)

  if(nrow(style_id_df)){
    as_xml_node(glue("<a:tableStyleId>{style_id_df[['style_id']]}</a:tableStyleId>"))
  }else{
    cli_abort(c(
      "Invalid Table Style Provided.",
      i = paste0("Viable Table Styles include: ", paste0("`",unique(table_style_df$style_name),"`", collapse = ", ")),
      x = paste0("Invalid Table Style provided: `",table_style_name,"`")
    ), call = error_call)
    return(NULL)
  }

}

#' @importFrom dplyr bind_rows
#' @importFrom tibble tibble
#' @importFrom purrr map
get_table_styles <- function(pptx){

  table_style_list_xml <- read_table_style_file(pptx)

  styles <- xml_find_all(table_style_list_xml, "//a:tblStyle")

  if( length(styles) == 0 ){
    return(data.frame(style_id = character(), style_name = character()))
  }else{
    styles |>
      map(function(x) {
        tibble(style_id = xml_attr(x, "styleId"),
               style_name = xml_attr(x, "styleName"))
      }) |>
      bind_rows()
  }
}

#' @importFrom xml2 read_xml
read_table_style_file <- function(pptx){

  table_style_xml_file <- file.path(pptx$rpptx$package_dir, "ppt/tableStyles.xml")

  if(!file.exists(table_style_xml_file)){
    table_style_list_xml <- read_xml(
      '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
      <a:tblStyleLst xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main">
      </a:tblStyleLst>
      ')

  }else{
    table_style_list_xml <- read_xml(table_style_xml_file)
  }

  table_style_list_xml
}

#' Polish content
#'
#' @param x object to be polished into ooxml
#' @param type output type. Valid values are "word" and "pptx".
#' @param ... additional arguments for methods
#' @param error_class Additional error class.
#' @inheritParams rlang::args_error_context
#'
#' @details depending on `type` either the [polish_content_pptx()] or [polish_content_word()]
#'          function is called.
#'
#' @returns  an object of class "polish_xml_nodeset". Contains XML that is viable Word or PowerPoint OOXML for addition
#'   to Word documents or PowerPoint presentations based on the "type".
#'
#' @export
polish_content <- function(x, type, ..., error_call = current_env(), error_class = NULL) {
  if (missing(type)) {
    cli::cli_abort(
      '{.arg type} must be one of "word" or "pptx", not absent.',
      call = error_call, class = c(error_class, "polish_content_error"))
  }
  type <- arg_match(type, values = c("word", "pptx"), error_call = error_call)

  polished <- withCallingHandlers({
    switch(
      type,
      word = polish_content_word(x, ...),
      pptx = polish_content_pptx(x, ...)
    )
  }, error = function(err) {
      cli_abort(
        "Cannot polish content for {.emph {type}} documents.",
        parent = err,
        call = error_call,
        class = c(error_class, glue("polish_content_{type}_error"), "polish_content_error")
      )
    }
  )

  if (!inherits(polished, "xml_nodeset")) {
    dispatch <- switch(type,
      word = s3_dispatch(polish_content_word(x, ...)),
      pptx = s3_dispatch(polish_content_pptx(x, ...))
    )
    bullets <- set_names(capture.output(print(dispatch)), " ")

    bullets <- c(
      "Polished content must be an {.cls xml_nodeset} object, not {.obj_type_friendly {polished}}.",
      i = "Check the `polish_content_{type}()` method objects of class {.cls {x}}.",
      bullets
    )

    cli_abort(
      bullets, call = error_call,
      class = c(error_class, "polish_content_{type}_result_error", "polish_content_result_error")
    )
  }

  polished
}

abort_default <- function(x, error_call = caller_env()) {
  cli_abort("No available method for objects of type {.cls {class(x)}}.", call = error_call)
}

abort_unknown_extension <- function(x, error_call = caller_env()) {
  ext <- file_extension(x)
  cli_abort("No available method for `.{ext}` files.", call = error_call)
}

test_that("polish_content() fails with wrong type", {
  expect_snapshot(error = TRUE, cnd_class = TRUE, polish_content("", type = "xml"))
})

test_that("polish_content() fails on unknown class", {
  foo <- new_object("foo")
  expect_snapshot(error = TRUE, cnd_class = TRUE, polish_content(foo, type = "word"))
  expect_snapshot(error = TRUE, cnd_class = TRUE, polish_content(foo, type = "pptx"))
})

test_that("polish_content(type=) is mandatory", {
  expect_snapshot(error = TRUE, polish_content(42))
})

test_that("polish_content() fails on files with unknown extensions", {
  file <- local_file(".foo")
  expect_snapshot(error = TRUE, cnd_class = TRUE, polish_content(file, type = "word"))
  expect_snapshot(error = TRUE, cnd_class = TRUE, polish_content(file, type = "pptx"))
})

test_that("polish_content() checks that the method returns an xml_nodeset", {
  local_methods(
    polish_content_word.foo = function(x, ..., error_call = current_env()) {
      "not an xml_nodeset"
    },
    polish_content_word.bar = function(x, ..., error_call = current_env()) {
      NextMethod()
    },

    polish_content_pptx.foo = function(x, ph = '<p:ph/>', ..., error_call = current_env()) {
      "not an xml_nodeset"
    },
    polish_content_pptx.bar = function(x, ph = '<p:ph/>', ..., error_call = current_env()) {
      NextMethod()
    }
  )

  foo <- structure(list(), class = "foo")
  bar <- structure(list(), class = c("bar", "foo"))
  expect_snapshot(error = TRUE, polish_content(foo, type = "word"))
  expect_snapshot(error = TRUE, polish_content(bar, type = "word"))

  expect_snapshot(error = TRUE, polish_content(foo, type = "pptx"))
  expect_snapshot(error = TRUE, polish_content(bar, type = "pptx"))
})

test_that("polish_content() methods have a error_call= argument", {
  expect_have_argument("^polish_content_word[.]", "error_call")
  expect_have_argument("^polish_content_pptx[.]", "error_call")
})

test_that("polish_content_pptx() methods have a ph= argument", {
  expect_have_argument("^polish_content_pptx[.]", "ph")
})

test_that("polish_content_pptx() methods have a pptx= argument", {
  expect_have_argument("^polish_content_pptx[.]", "pptx")
})

test_that("polish_content_word() methods have a inline= argument", {
  expect_have_argument("^polish_content_word[.]", "inline")
})

#' Mark as a node to inject as is
#'
#' @param x raw ooxml passed through [as_xml_node()]
#'
#' @inheritParams rlang::args_error_context
#'
#' @returns adds the class "as_is" to the input text the content "as is" and not
#'   modify it when polishing
#'
#' @export
as_is <- function(x, error_call = current_env()) {
  add_class(x, "as_is")
}

#' Mark text as markdown
#'
#' @param x text to be interpreted as markdown
#'
#' @returns adds the class "as_ms" to the input text the content "as markdown"
#'   apply markdown formatting to the text when polishing
#'
#' @export
as_md <- function(x) {
  add_class(x, "as_md")
}

add_class <- function(x, class = NULL) {
  class(x) <- c(class, setdiff(class(x), class))
  x
}

test_that("as_is()", {
  expect_equal(
    polish_content_word(as_is("<w:p/>")),
    as_xml_word("<w:p/>")
  )
  expect_equal(
    polish_content_word(as_is("<w:tbl/>")),
    as_xml_word("<w:tbl/>")
  )

  expect_equal(
    polish_content_word(as_is("<w:p/><w:p/>")),
    as_xml_word("<w:p/><w:p/>")
  )

  expect_equal(
    polish_content_word(as_is("<w:tbl/><w:p/>")),
    as_xml_word("<w:tbl/><w:p/>")
  )

  expect_snapshot(error = TRUE,
    polish_content_word(as_is("<a/>"))
  )

  expect_snapshot(error = TRUE,
    polish_content_word(as_is("<a"))
  )

  expect_snapshot(error = TRUE,
    polish_content_word(as_is("<a/><w:p/>"))
  )

  expect_snapshot(error = TRUE,
    polish_content_word(as_is("<a/><w:p/><b/>"))
  )
})

test_that("lists can be converted into bulleted lists - pptx", {

  simple_list <- list("These","are","separate","bullets")
  nested_list <- list("Non indented bullet 1",list("indentet bullet 1","indented bullet 2"),"Non indented bullet 2")

  ## basic list
  expect_snapshot(
    polish_content_pptx.list(simple_list)
  )

  ## basic ordered list
  expect_snapshot(
    polish_content_pptx.list(simple_list, list_type = "ordered")
  )

  ## nested list
  expect_snapshot(
    polish_content_pptx.list(nested_list)
  )

  ## nested ordered list
  expect_snapshot(
    polish_content_pptx.list(nested_list, list_type = "ordered")
  )

})

test_that("lists can be converted into indented text if content is text, numeric, or md - pptx", {

  success_list <- list("These",1,as_md("**hello**"))

  expect_snapshot(
    polish_content_pptx.list(success_list, list_type = "none")
  )

})

test_that("lists can be converted into bulleted lists if content is text, numeric, or md - pptx", {

  success_list <- list("These",1,as_md("**hello**"))

  expect_snapshot(
    polish_content_pptx.list(success_list, list_type = "unordered")
  )

})

test_that("lists can be converted into bulleted lists, apply styling - pptx", {

  simple_list <- list("These","are","separate","bullets")

  ## basic list
  expect_snapshot(
    polish_content_pptx.list(
      simple_list,
      font_color = "blue",
      font_style = "italic",
      font_size = 20
      )
  )

})

test_that("lists cannot be converted into bulleted lists if content is not text, numeric, or md - pptx", {

  expect_snapshot(
    polish_content_pptx.list(list(mtcars)),
    error = TRUE
  )

})

#' Polish content for Word
#'
#' @param x object to be polished
#' @param ... used in some methods
#' @param inline if `FALSE` the comments are wrapped in a `<w:p>`
#' @inheritParams rlang::args_error_context
#' @returns  an object of class "polish_xml_nodeset". Contains XML that is viable Word OOXML for addition to
#'   Word documents.
#' @export
polish_content_word <- function(x, inline = FALSE, ..., error_call = current_env()) {
  UseMethod("polish_content_word")
}

#' @export
polish_content_word.default <- function(x, inline = FALSE, ..., error_call = current_env()) {
  abort_default(x, error_call = error_call)
}

#' @export
polish_content_word.file <- function(x, inline = FALSE, ..., error_call = current_env()) {
  abort_unknown_extension(x, error_call = error_call)
}

#' @inheritParams rlang::args_error_context
#' @rdname polish_content_word
#' @param font_color color of text. Enter as a hex code.
#' @param font_style styling of text, vector including "bold","italic","underline".
#' @param font_size point size of text if different from document default.
#' @param font_typeface typeface of the text to be added if different from document default.
#'
#' @export
polish_content_word.character <- function(x, inline = FALSE, ..., font_color = NULL, font_style = NULL, font_size = NULL, font_typeface = NULL, error_call = current_env()) {
  polish_check_dots_empty(call = error_call)
  inline <- isTRUE(inline)

  if (length(x) > 1 && inline) {
    cli_warn(call = error_call, c(
      "Polishing a vector containing more than one element with {.code inline = TRUE}.",
      i   = "The elements are combined without spaces.",
      i   = "This might lead to unexpected output."
    ))
  }

  font_style_xml <- style_text_word(font_color, font_style, font_size, font_typeface)

  xml_content <- glue('
    <w:r>
      {font_style_xml}
      <w:t xml:space="preserve">{x}</w:t>
    </w:r>
  ')
  xml_content <- wrap_inline(xml_content, inline)
  xml_content <- glue_collapse(xml_content, sep = "\n")
  as_xml_word(xml_content, error_call = error_call)
}

#' @export
polish_content_word.numeric <- function(x, inline = FALSE, ..., error_call = current_env()) {
  polish_content_word(format(x, ...), inline = inline, error_call = error_call)
}

#' @export
polish_content_word.as_is <- function(x, inline = TRUE, ..., valid = c("w:p", "w:tbl"), error_call = current_env()) {
  x <- wrap_inline(x, inline)
  node <- as_xml_word(x, error_call = error_call)

  node_name <- xml_name(node, ns = xml_ns(node))
  if (sum(!node_name %in% valid) > 0) {

    invalid_node_names <- paste(unique(node_name[!node_name %in% valid]),sep = ",")

    valid <- glue_collapse(glue("<{valid}>"), sep = ", ", last = " or ")
    invalid <- glue_collapse(glue("<{invalid_node_names}>"), sep = ", ", last = " or ")

    cli_abort(
      "{.arg x} must be a {.emph {valid}} node, not {.emph {invalid}}.",
      call = error_call
    )
  }

  node
}

#' @export
polish_content_word.as_md <- function(x, inline = TRUE, ..., error_call = current_env()) {
  polish_check_dots_empty(call = error_call)

  x <- wrap_inline(x, inline)
  glue_collapse(x, sep = "\n") |>
    gt::vec_fmt_markdown(output = "word") |>
    as_xml_word(error_call = error_call) |>
    xml_child() |>
    as_xml_word(error_call = error_call)
}

#' @export
polish_content_word.data.frame <- function(x, inline = TRUE, ..., error_call = current_env()) {
  polish_check_dots_empty(call = error_call)

  bt <- officer::block_table(x, header = TRUE, alignment = "center")
  xml <- officer::to_wml(bt, add_ns = TRUE)
  xml <- wrap_inline(xml, inline)
  as_xml_word(xml, error_call = error_call)
}


# file_ -------------------------------------------------------------------

#' @param width,height,units Plot size in `units` ("in", "cm", "mm", or "px").
#'   defaults to 5 inches tall and 6 inches wide.
#' @rdname polish_content_word
#' @export
polish_content_word.file_png <- function(x, inline = FALSE, ..., height = 5, width = 6, units = "in", error_call = current_env()) {
  polish_check_dots_empty(call = error_call)

  ext_img <- officer::external_img(x, height = height, width = width, unit = units)
  xml_content <- officer::to_wml(ext_img, add_ns = TRUE)
  xml_content <- wrap_inline(xml_content, inline)
  as_xml_word(xml_content, error_call = error_call)
}

#' @export
polish_content_word.file_jpeg <- polish_content_word.file_png

#' @export
polish_content_word.file_svg <- polish_content_word.file_png

#' @export
polish_content_word.file_txt <- function(x, inline = FALSE, ..., error_call = current_env()) {
  polish_check_dots_empty(call = error_call)

  x_content <- iconv(readLines(x), to = "UTF-8")
  xml_content <- glue_collapse(
    glue('
    <w:r>
      <w:t>{x_content}</w:t>
    </w:r>
    '),
    sep = '<w:br w:type="textWrapping" w:clear="all"></w:br>'
  )

  xml_content <- wrap_inline(xml_content, inline)
  as_xml_word(xml_content, error_call = error_call)
}

#' @export
polish_content_word.file_md <- function(x, inline = TRUE, ..., error_call = current_env()){
  polish_check_dots_empty(call = error_call)
  polish_content_word(as_md(readLines(x)), error_call = error_call)
}

#' @export
polish_content_word.file_docx <- function(x, inline = TRUE, ..., error_call = current_env()){
  polish_check_dots_empty(call = error_call)

  bt <- officer::block_pour_docx(file = x)
  xml_content <- officer::to_wml(bt, add_ns = TRUE)
  xml_content <- wrap_inline(xml_content, inline)
  as_xml_word(xml_content, error_call = error_call)
}

#' @export
polish_content_word.file_rtf <- function(x, inline = TRUE, ..., error_call = current_env()){
  polish_check_dots_empty(call = error_call)

  path <- normalizePath(x, winslash = "/", mustWork = FALSE)
  xml_content <- glue('<w:altChunk r:id="{path}"/>')
  xml_content <- wrap_inline(xml_content, inline)
  as_xml_word(xml_content, error_call = error_call)
}

#' @export
polish_content_word.file_html <- function(x, inline = TRUE, ..., error_call = current_env()){
  polish_check_dots_empty(call = error_call)

  path <- normalizePath(x, winslash = "/", mustWork = FALSE)
  xml_content <- glue('<w:altChunk r:id="{path}"/>')
  xml_content <- wrap_inline(xml_content, inline)
  as_xml_word(xml_content, error_call = error_call)
}

## defined placeholders
ph_dims_unset <- "<p:nvSpPr><p:cNvPr id=\"c5672246-4915-4812-b4a2-e7de4805bade\" name=\"unset dims Placeholder\"/><p:cNvSpPr><a:spLocks noGrp=\"1\"/></p:cNvSpPr><p:nvPr><p:ph type=\"body\"/></p:nvPr></p:nvSpPr><p:spPr></p:spPr>"
ph_dims_set <- "<p:nvSpPr><p:cNvPr id=\"c5672246-4915-4812-b4a2-e7de4805bade\" name=\"set dims  Placeholder\"/><p:cNvSpPr><a:spLocks noGrp=\"1\"/></p:cNvSpPr><p:nvPr><p:ph type=\"body\"/></p:nvPr></p:nvSpPr><p:spPr><a:xfrm><a:off x=\"487680\" y=\"480060\"/><a:ext cx=\"7071360\" cy=\"5829300\"/></a:xfrm></p:spPr>"

test_that("polish_content_word().ggplot - undefined height and width", {
  suppressWarnings(withr::local_package("ggplot2"))

  p <- ggplot(mtcars, aes(mpg, cyl)) + geom_point()

  ## provided ph with out set dims/location
  xml_dims_unset <- polish_content_pptx(p, ph = ph_dims_unset)

  expect_equal(xml_attr(xml_find_all(xml_dims_unset, ".//a:off"), "x"), "0")
  expect_equal(xml_attr(xml_find_all(xml_dims_unset, ".//a:off"), "y"), "0")
  expect_equal(xml_attr(xml_find_all(xml_dims_unset, ".//a:ext"), "cx"), "5486400")
  expect_equal(xml_attr(xml_find_all(xml_dims_unset, ".//a:ext"), "cy"), "4572000")

  ## provided ph with set dims/location
  xml_dims_set <- polish_content_pptx(p, ph = ph_dims_set)

  expect_equal(xml_attr(xml_find_all(xml_dims_set, ".//a:off"), "x"), "487680")
  expect_equal(xml_attr(xml_find_all(xml_dims_set, ".//a:off"), "y"), "480060")
  expect_equal(xml_attr(xml_find_all(xml_dims_set, ".//a:ext"), "cx"), "7071360")
  expect_equal(xml_attr(xml_find_all(xml_dims_set, ".//a:ext"), "cy"), "5829300")

})

test_that("polish_content_word().ggplot - defined height and width,", {

  suppressWarnings(withr::local_package("ggplot2"))

  p <- ggplot(mtcars, aes(mpg, cyl)) + geom_point()

  ## provided ph with out set dims/location
  xml_dims_unset <- polish_content_pptx(p, ph = ph_dims_unset, height = 10, width = 12)

  expect_equal(xml_attr(xml_find_all(xml_dims_unset, ".//a:off"), "x"), "0")
  expect_equal(xml_attr(xml_find_all(xml_dims_unset, ".//a:off"), "y"), "0")
  expect_equal(xml_attr(xml_find_all(xml_dims_unset, ".//a:ext"), "cx"), "10972800")
  expect_equal(xml_attr(xml_find_all(xml_dims_unset, ".//a:ext"), "cy"), "9144000")

  ## provided ph with set dims/location, image dims smaller than ph, image ratio of .83333
  xml_dims_set <- polish_content_pptx(p, ph = ph_dims_set, height = 2, width = 2.4)

  expect_equal(xml_attr(xml_find_all(xml_dims_set, ".//a:off"), "x"), "525780")
  expect_equal(xml_attr(xml_find_all(xml_dims_set, ".//a:off"), "y"), "480060")
  expect_equal(xml_attr(xml_find_all(xml_dims_set, ".//a:ext"), "cx"), "6995160")
  expect_equal(xml_attr(xml_find_all(xml_dims_set, ".//a:ext"), "cy"), "5829300")

  ## provided ph with set dims/location, image dims larger than ph, image ratio of .83333
  xml_dims_set <- polish_content_pptx(p, ph = ph_dims_set, height = 10, width = 12)

  expect_equal(xml_attr(xml_find_all(xml_dims_set, ".//a:off"), "x"), "525780")
  expect_equal(xml_attr(xml_find_all(xml_dims_set, ".//a:off"), "y"), "480060")
  expect_equal(xml_attr(xml_find_all(xml_dims_set, ".//a:ext"), "cx"), "6995160")
  expect_equal(xml_attr(xml_find_all(xml_dims_set, ".//a:ext"), "cy"), "5829300")

})

test_that("polish_content_word().ggplot - partially or incorrectly defined height and width,", {

  suppressWarnings(withr::local_package("ggplot2"))

  p <- ggplot(mtcars, aes(mpg, cyl)) + geom_point()

  expect_snapshot(
    polish_content_pptx(p, ph = ph_dims_unset, height = NA, width = 12),
    error = TRUE
  )

  expect_snapshot(
    polish_content_pptx(p, ph = ph_dims_unset, height = NULL, width = 12),
    error = TRUE
  )

  expect_snapshot(
    polish_content_pptx(p, ph = ph_dims_unset, height = -10, width = 12),
    error = TRUE
  )

  expect_snapshot(
    polish_content_pptx(p, ph = ph_dims_unset, height = c(10,11), width = 12),
    error = TRUE
  )

  expect_snapshot(
    polish_content_pptx(p, ph = ph_dims_unset, height = "invalid", width = 12),
    error = TRUE
  )

})

test_that("polish_content_word.as_md()", {
  expect_snapshot(
    polish_content_word(as_md("*hello*"))
  )
})


test_that("polish_content_pptx.as_md()", {
  expect_snapshot(
    polish_content_pptx(as_md("*hello*"))
  )

  expect_snapshot(
    polish_content_pptx(as_md("__hello__"))
  )

  expect_snapshot(
    polish_content_pptx(as_md("## hello"))
  )

  ## List markdowns ----
  ### Unordered ----
  expect_snapshot(
    polish_content_pptx(as_md("- hello"))
  )

  expect_snapshot(
    polish_content_pptx(as_md("- hello\n- world!"))
  )

  expect_snapshot(
    polish_content_pptx(as_md("- hello\n- *world*!"))
  )

  expect_snapshot(
    polish_content_pptx(as_md("- hello\n    - world!"))
  )

  expect_snapshot(
    polish_content_pptx(as_md("- hello\n    - *world*!"))
  )

  ### Ordered----
  expect_snapshot(
    polish_content_pptx(as_md("1. hello"))
  )

  expect_snapshot(
    polish_content_pptx(as_md("1. hello\n2. world!"))
  )

  expect_snapshot(
    polish_content_pptx(as_md("1. hello\n2. *world*!"))
  )

  expect_snapshot(
    polish_content_pptx(as_md("1. hello\n    1. world!"))
  )

  expect_snapshot(
    polish_content_pptx(as_md("1. hello\n    1. *world*!"))
  )


  ##  links
  expect_snapshot(
    polish_content_pptx(as_md("markdown is defined at [this website](https://daringfireball.net/projects/markdown/)"))
  )

})

test_that("Expected Errors",{


  ## Image
  expect_snapshot(
    polish_content_pptx(as_md("![subtext](man/figures/logo.png)")),
    error = TRUE
  )

})

test_that("polish_content_word(<gt_tbl>", {
  suppressWarnings(withr::local_package("gt"))

  tab <- gt(gtcars) |>
    tab_header(
      title = md("Data listing from **gtcars**"),
      subtitle = md("`gtcars` is an R dataset")
    )

  xml <- polish_content_word(tab)
  expect_equal(
    xml_name_word(xml),
    c("w:p", "w:p", "w:tbl")
  )

  # inline= is just ignored
  xml <- polish_content_word(tab, inline = TRUE)
  expect_equal(
    xml_name_word(xml),
    c("w:p", "w:p", "w:tbl")
  )
})

test_that("polish_content_word(<gt_group>", {
  suppressWarnings(withr::local_package("gt"))

  tab <- gt(gtcars) |>
    tab_header(
      title = md("Data listing from **gtcars**"),
      subtitle = md("`gtcars` is an R dataset")
    )

  tab2 <- gt(gtcars) |>
    tab_header(
      title = md("Data listing from **gtcars** second table"),
      subtitle = md("`gtcars` is an R dataset")
    )

  tab_group <- gt_group(tab, tab2)

  xml <- polish_content_word(tab_group)
  expect_equal(
    xml_name_word(xml),
    c("w:p", "w:p", "w:tbl","w:p","w:p", "w:p", "w:tbl")
  )

  # inline= is just ignored
  xml <- polish_content_word(tab_group, inline = TRUE)
  expect_equal(
    xml_name_word(xml),
    c("w:p", "w:p", "w:tbl", "w:p","w:p", "w:p", "w:tbl")
  )
})

test_that("polish_content_word() fails on unknown class", {
  expect_snapshot(error = TRUE,
    polish_content_word(new_object("foo"))
  )
})

test_that("polish_content_word() fails on files with unknown extensions", {
  expect_snapshot(error = TRUE,
    polish_content_word(local_file(".foo"))
  )
})

test_that("polish_content_word.character()", {
  txt <- "   my character value  "
  xml <- polish_content_word(txt)
  expect_equal(length(xml_find_all(xml, "//w:p")), 1L)
  expect_equal(xml_text(xml_find_all(xml, "//w:p/w:r/w:t")), txt)
  expect_equal(length(xml_find_all(xml, "//w:p/w:r/w:rPr")), 0L)

  txt <- c("   my character value  ","char value 2")
  xml <- polish_content_word(txt)
  expect_equal(length(xml_find_all(xml, "//w:p")), 2L)
  expect_equal(xml_text(xml_find_all(xml, "//w:p/w:r/w:t")), txt)
  expect_equal(length(xml_find_all(xml, "//w:p/w:r/w:rPr")), 0L)

  txt <- c("hello", "world")
  expect_warning(
    xml <- polish_content_word(c("hello", "world"), inline = TRUE)
  )
  expect_equal(xml_name(xml), c("r", "r"))
  expect_equal(xml_text(xml_find_all(xml, "//w:r/w:t")), txt)

  txt <- "   my character value  "
  xml <- polish_content_word(txt, font_color = "red", font_size = 12)
  expect_equal(length(xml_find_all(xml, "//w:p")), 1L)
  expect_equal(xml_text(xml_find_all(xml, "//w:p/w:r/w:t")), txt)
  w_rPr <- xml_find_all(xml, "//w:p/w:r/w:rPr")
  expect_equal(length(w_rPr), 1L)
  expect_equal(xml_attr(xml_find_all(w_rPr, "w:sz"), "val"), "12")
  expect_equal(xml_attr(xml_find_all(w_rPr, "w:color"), "val"), "FF0000")


  xml <- polish_content_word(txt, font_typeface = "helvetica", font_size = 12, font_style = c("bold","italic"))
  expect_equal(length(xml_find_all(xml, "//w:p")), 1L)
  expect_equal(xml_text(xml_find_all(xml, "//w:p/w:r/w:t")), txt)
  w_rPr <- xml_find_all(xml, "//w:p/w:r/w:rPr")
  expect_equal(length(w_rPr), 1L)
  expect_equal(xml_attr(xml_find_all(w_rPr, "w:sz"), "val"), "12")
  expect_equal(xml_attr(xml_find_all(w_rPr, "w:rFonts"), "ascii"), "helvetica")
  expect_equal(xml_attr(xml_find_all(w_rPr, "w:rFonts"), "hAnsi"), "helvetica")
  expect_equal(xml_attr(xml_find_all(w_rPr, "w:rFonts"), "cs"), "helvetica")
  expect_equal(xml_name(xml_children(w_rPr)), c("rFonts", "sz", "b", "i"))

  expect_snapshot(error = TRUE,
    polish_content_word("<")
  )
})

test_that("polish_content_word.character() tolerates non empty ...", {
  local_options(polish.dots_error = FALSE)
  expect_no_error(polish_content_word("hello", foo = TRUE))
})

test_that("polish_content_word.data.frame()", {
  expect_snapshot(
    polish_content_word(data.frame(x = 1:2, y = c("hello", "world")))
  )
})

test_that("polish_content_word().numeric", {
  expect_snapshot(
    polish_content_word(42L)
  )
  expect_snapshot(
    polish_content_word(42.5)
  )

  expect_snapshot(
    polish_content_word(c(42L, 43L))
  )
  expect_snapshot(
    polish_content_word(c(42.5, 43.5))
  )
})

test_that("polish_content_word().ggplot", {
  suppressWarnings(withr::local_package("ggplot2"))

  expect_snapshot(transform = transform_polish,
    polish_content_word(ggplot(mtcars))
  )
})

convert_rtf_file_to_png <- function(path, ..., temp_dir = tempfile(), error_call = current_env()){
  check_dots_empty(call = error_call)
  ensure_dir_exists(temp_dir)

  tmp_ref <- file.path(temp_dir, change_extension(path, "html"))

  rmarkdown::pandoc_convert(
    input = normalizePath(path, winslash = "/"),
    output = tmp_ref,
    verbose = FALSE
  )

  convert_html_file_to_png(tmp_ref, temp_dir = temp_dir)
}

convert_html_file_to_png <- function(path, ..., temp_dir = tempfile(), error_call = current_env()){
  check_dots_empty(call = error_call)
  ensure_dir_exists(temp_dir)

  tmp_png <- file.path(temp_dir, change_extension(path, "png"))

  path <- fit_body_to_content(path)

  webshot2::webshot(
    url = paste0("file:///", path),
    file = tmp_png,
    selector = "body",
    quiet = TRUE,
    zoom = 2
  )

  tmp_png
}

convert_pdf_file_to_png <- function(path, ..., temp_dir = tempfile(), error_call = current_env()){
  check_dots_empty(call = error_call)
  ensure_dir_exists(temp_dir)

  tmp_png <- file.path(temp_dir, change_extension(path, "png"))

  suppressMessages({
    image <- magick::image_read_pdf(path)
    converted_image <- magick::image_convert(image, format = "png")
    magick::image_write(converted_image, path = tmp_png)
    gc(verbose = FALSE)
  })

  tmp_png
}

fit_body_to_content <- function(x, temp_dir = tempfile()){
  ensure_dir_exists(temp_dir)

  # make body fit-content
  html_content <- xml2::read_html(as.character(x))

  body <- xml_find_first(html_content,"body")
  xml_attr(body,"style")  <- "width: fit-content;height: fit-content;"

  tmp_html <- tempfile(tmpdir = temp_dir, pattern = file_path_sans_ext(basename(x)), fileext = ".html")
  xml2::write_html(html_content, tmp_html)
  tmp_html
}

list_as_ppt_xml <- function(x,
                            list_type = "none",
                            level = 0,
                            escape = TRUE,
                            collapse = FALSE,
                            font_color = NULL,
                            font_style = NULL,
                            font_size = NULL,
                            font_typeface = NULL,
                            ...,
                            error_call = current_env()){


  map(x,function(x_item){

    if(is_list(x_item)){

      ## allow for nesting lists
      output_content <- list_as_ppt_xml(
          x_item,
          list_type = list_type,
          level = level + 1,
          escape = escape,
          collapse = collapse,
          font_color = font_color,
          font_style = font_style,
          font_size = font_size,
          font_typeface = font_typeface,
          error_call = error_call
      )

    }else{

      if(inherits(x_item,"as_md")){
        output_content <- md_as_ppt_xml(x_item, level = level)
      }else if(inherits(x_item,c("numeric","character"))){

        if(is.numeric(x_item)){
          x_item <- format(x_item)
        }

        run_content <- glue_collapse(
          polish_pptx_strings(
            x_item,
            escape = escape,
            collapse = collapse,
            font_color = font_color,
            font_style = font_style,
            font_size = font_size,
            font_typeface = font_typeface,
            error_call = error_call
          ))

          pPr <- c(
            none = paste0(
              '<a:pPr marL="',
              514350 * (1 + level),
              '" indent="-514350"',
              '/>'
            ),
            ## if bulleted, only start adding pPr if more than 1 level
            unordered = paste0(
              '<a:pPr marL="',
              514350 * (1 + level),
              '" indent="-514350"',
              ifelse(level == 0, "", paste0(' lvl="', level, '"')),
              '><a:buFont typeface="Arial" panose="020B0604020202020204" pitchFamily="34" charset="0"/>',
              '<a:buChar char="\u2022"/></a:pPr>'
              ),
            ## if ordered, start adding lvl if more than 1 layer. set indent and margin-left
            ordered = paste0(
              '<a:pPr marL="',
              514350 * (1 + level),
              '" indent="-514350"',
              ifelse(level == 0, "", paste0(' lvl="', level, '"')),
              '><a:buFont typeface="+mj-lt"/><a:buAutoNum type="arabicPeriod"/></a:pPr>'
            )
          )[[list_type]]

          output_content <- glue('<a:p>{pPr}{run_content}</a:p>')

      }else{
        cli_abort("Cannot polish a list with elements of type {.cls {class(x_item)}}.", call = error_call)
      }

    }


    output_content

  }) |>

    unlist()

}




check_all_list_viable <- function(x, suppress_error = FALSE, error_call = current_env()){

  res_all <- map(x,function(z){

    if(is.list(z)){
      res <- check_all_list_viable(z, suppress_error = TRUE)
    }else{
      res <- is.character(z) | is.numeric(z) & length(z) == 1
    }
    res
  }) |> unlist()

  is_viable <- sum(!res_all) == 0

  if(!is_viable & !suppress_error){
    cli_abort(
      "List objects to be polished can only contain character or numeric values of length 1",
      call = error_call
    )
  }

  invisible(is_viable)

}

# word --------------------------------------------------------------------

#' @export
polish_content_word.gt_tbl <- function(x, inline = FALSE, keep_with_next = TRUE, autonum = TRUE, ..., error_call = current_env()) {
  polish_check_dots_empty(call = error_call)

  xml <- gt::as_word(x, keep_with_next = keep_with_next, autonum = autonum)
  as_xml_word(xml, error_call = error_call)
}

#' @export
polish_content_word.gt_group <- function(x, inline = FALSE, keep_with_next = TRUE, autonum = TRUE, ..., error_call = current_env()) {
  polish_check_dots_empty(call = error_call)

  xml_list <- list()
  for(i in 1:nrow(x$gt_tbls)){
    xml <- gt::as_word(grp_pull(x,i), keep_with_next = keep_with_next, autonum = autonum)
    xml_list[[length(xml_list)+1]] <- xml

    # Add empty line between the tables
    if(i<nrow(x$gt_tbls)){
      xml_list[[length(xml_list)+1]] <- "<w:p/>"
    }
  }
  as_xml_word(xml_list, error_call = error_call)
}

#' @param res Plot resolution. Also accepts a string input: "retina" (320),
#'   "print" (300), or "screen" (72). Applies only to raster output types.
#' @rdname polish_content_word
#' @export
polish_content_word.ggplot <- function(x, inline = FALSE, ..., height = 5, width = 6, units = "in", res = 300, error_call = current_env()){
  png <- polish_ggplot(x, height = height, width = width, units = units, dpi = res)
  polish_content_word(png, inline = inline, ..., height = height, width = width, units = "in", error_call = error_call)
}

#' @param width,height defaults to match the height and width of the placeholder
#'   provided. When that cannot be derived, it assumes the plot should be 5
#'   inches tall and 6 inches wide. Users may set the width and height, but they
#'   most both be set.
#' @param res Plot resolution. Also accepts a string input: "retina" (320),
#'   "print" (300), or "screen" (72). Applies only to raster output types.
#' @param device what graphic device to use when saving the plot. Can be one of
#'   "png","svg", or "jpeg".
#'
#' @param scale Multiplicative scaling factor.
#' @rdname polish_content_pptx
#' @export
polish_content_pptx.ggplot <- function(x, ph = "<p:ph/>", pptx, ..., height = NULL, width = NULL, res = 300, device = c("png","svg","jpeg"), scale = 1, error_call = current_env()){

  ## get width and height of placeholder if not set
  if(is.null(height) & is.null(width)){

    # wrap ph in a <p:sp> node
    sp_ph      <- sp_shell(ph)

    if (length(xml_find_all(sp_ph, ".//a:xfrm")) == 1) {
      ## gives dimensions of the placehlder in inches
      dims    <- dim_extract(sp_ph, ".//a:ext", c("cx", "cy"))
      height  <- dims$cy
      width   <- dims$cx
      image_fit <- "stretch"
    }else{
      height <- 5
      width  <- 6
      image_fit <- "scale"
    }
  }else{

    invalid_dims <- is.null(height) | is.null(width)
    if(!is.null(width) & !invalid_dims){
      invalid_dims <- !is.numeric(width) | (length(width) > 1) | isTRUE(width <= 0)
    }
    if(!is.null(height) & !invalid_dims){
      invalid_dims <- !is.numeric(height) | (length(height) > 1) | isTRUE(height <= 0)
    }

    if(invalid_dims){

      width_value <- width_value %||% "NULL"
      height_value <- height %||% "NULL"

      cli_abort(c(
        "{.arg height} and {.arg width} must both be set if defining the plot size, and be positive numeric values.",
        i="You've supplied for {.arg height} a {.cls {class(height)}} vector with value {.val {height_value}}.",
        i="You've supplied for {.arg width} a {.cls {class(width)}} vector with value {.val {width_value}}."
      ), call = error_call)
    }

    image_fit <- "scale"
  }

  png <- polish_ggplot(x, device = device, height = height, width = width, units = "in", dpi = res, scale = scale)
  polish_content_pptx(png, ph = ph, pptx = pptx, ..., height = height, width = width, units = "in", image_fit = image_fit, error_call = error_call)
}

polish_ggplot <- function(x, device = c("png","svg","jpeg"), ...) {
  device <- match.arg(device)
  tmp_img <- tempfile(fileext = paste0(".", device))
  ggsave(x, filename = tmp_img, device = device, ...)
  as_file(tmp_img)
}

style_text_word <- function(font_color = NULL, font_style = NULL, font_size = NULL, font_typeface = NULL){

  if (is.null(font_color) && is.null(font_style) && is.null(font_size) && is.null(font_typeface)) {
    return("")
  }

  typeface <- font_typeface_word(font_typeface)
  size     <- font_size_word(font_size)
  style    <- style_xml_word(font_style)
  color    <- font_color_xml_word(font_color)

  glue("<w:rPr>{typeface}{size}{style}{color}</w:rPr>")

}

style_text_pptx <- function(font_color = NULL, font_style = NULL, font_size = NULL, font_typeface = NULL){

  if (is.null(font_color) && is.null(font_style) && is.null(font_size) && is.null(font_typeface)) {
    return("")
  }

  font_typeface_xml <- font_typeface_xml_pptx(font_typeface)
  font_point_args   <- font_point_xml_pptx(font_size)
  font_style_args   <- style_xml_pptx(font_style)
  font_color_xml    <- font_color_xml_pptx(font_color)

  glue("
    <a:rPr {font_point_args} {font_style_args}>
       {font_color_xml}
       {font_typeface_xml}
    </a:rPr>
  ")

}


# typeface and size -------------------------------------------------------

font_typeface_word <- function(font_typeface = NULL) {
  if (is.null(font_typeface)) {
    return("")
  }

  glue('<w:rFonts w:ascii="{font_typeface}" w:hAnsi="{font_typeface}" w:cs="{font_typeface}"/>')
}

font_size_word <- function(font_size = NULL) {
  if (is.null(font_size)) {
    return("")
  }

  glue('<w:sz w:val="{font_size}"/>')
}

font_typeface_xml_pptx <- function(font_typeface = NULL){
  xml <- ""
  if (!is.null(font_typeface)) {
    xml <- glue('<a:latin typeface="{font_typeface}"/>')
  }
  xml
}

font_point_xml_pptx <- function(font_size = NULL){
  if (is.null(font_size)) {
    return("")
  }

  if (font_size <= 200) {
    font_size <- font_size * 100
  }
  glue('sz = "{font_size}"')
}

# style -------------------------------------------------------

style_xml_word <- function(font_style = c("bold", "italic", "underline")){
  if (is.null(font_style)) {
    return("")
  }

  style <- rlang::arg_match(font_style, multiple = TRUE)
  code  <- purrr::map_chr(docx_style_codes[style], "code")

  w <- paste0('<w:', code, '/>')
  glue_collapse(w)
}

style_xml_pptx <- function(font_style){
  if (is.null(font_style)) {
    return("")
  }

  style <- rlang::arg_match(font_style, names(docx_style_codes), multiple = TRUE)

  code  <- purrr::map_chr(docx_style_codes[style], "code")
  value <- purrr::map_chr(docx_style_codes[style], "value")

  args <- glue('{code} = "{value}"')
  glue_collapse(args, " ")
}

docx_style_codes <- list(
  bold      = list(code = "b", value = "1"),
  italic    = list(code = "i", value = "1"),
  underline = list(code = "u", value = "sng")
)

# color -------------------------------------------------------

font_color_xml_word <- function(font_color){
  if (is.null(font_color)) {
    return("")
  }

  hex <- as_hex_codes(font_color)
  glue('<w:color w:val="{hex}"/>')
}

font_color_xml_pptx <- function(font_color){
  if (is.null(font_color)) {
    return("")
  }

  hex <- as_hex_codes(font_color)
  glue('<a:solidFill><a:srgbClr val="{hex}"/></a:solidFill>')
}

#' @importFrom grDevices col2rgb rgb
as_hex_codes <- function(x) {

  tryCatch({
    ## if hex already, return the hex
    if (grepl("^(#)", x) || grepl("^(#)*[0-9A-Fa-f]{6}$", x, perl = TRUE)) {
      x <- toupper(x)
    } else {
      font_colors <- col2rgb(x)
      x <- rgb(font_colors[1], font_colors[2], font_colors[3], maxColorValue = 255)
    }

    gsub("^(#)","",x)
  }, error = function(e) {
    # TODO: should this be an informative error ?
    ""
  })

}

#' @importFrom commonmark markdown_xml
md_as_ppt_xml <- function(x, level = 0){

  content <- read_xml(markdown_xml(x)) |>
    xml_children()

  content_xml <- lapply(
    content,
    function(content_node){
      node_type <- xml_name(content_node)
      apply_format_pptx_xml(node_type,content_node, level = level)
    })

  paste0(unlist(content_xml),collapse = "\n")

}

apply_format_pptx_xml <- function(node, x, ...){

  if(node %in% names(format_pptx_xml)){
    format_pptx_xml[[node]](x, ...)
  }else{
    format_pptx_xml[["missing"]](x, ...)
  }
}

format_pptx_xml <- list(

  missing = function(x, ...){

    content <-xml_children(x)
    lapply(
      content,
      function(content_node){
        node_type <- xml_name(content_node)
        apply_format_pptx_xml(node_type,content_node, ...)
      }) |>
      unlist()
  },

  list = function(x, ..., layer = 0){

    list_type <- xml_attr(x, "type") ## types are "ordered" or "bullet"
    list_items <- x |> xml_children()


    pPr <- c(
      ## if bulleted, only start adding pPr if more than 1 layer
      bullet = ifelse(layer == 0, "", paste0('<a:pPr lvl="', layer, '"/>')),
      ## if ordered, start adding lvl if more than 1 layer. set indent and margin-left
      ordered = paste0(
        '<a:pPr marL="',
        514350 * (1 + layer),
        '" indent="-514350"',
        ifelse(layer == 0, "", paste0(' lvl="', layer, '"')),
        '><a:buFont typeface="+mj-lt"/><a:buAutoNum type="arabicPeriod"/></a:pPr>'
      )
    )[[list_type]]

    lapply(
      list_items,
      function(item_node){
        content <-xml_children(item_node)
        lapply(
          content,
          function(content_node){
            node_type <- xml_name(content_node)
            apply_format_pptx_xml(node_type,content_node, pPr = pPr, layer = layer + 1)
          })
      }) |>
      unlist()

  },

  paragraph = function(x, ..., pPr = '<a:pPr marL="0" indent="0"><a:buNone/></a:pPr>') {
    paragraph_text <- x |> xml_find_all(".//d1:text")

    run_content <- c()

    for (txt in paragraph_text) {
      style_settings <- NULL
      link_path <- NULL
      image_path <- NULL
      parent_node <- xml2::xml_parent(txt)

      type <- "text"

      while (!identical(parent_node, x)) {
        parent_type <- xml_name(parent_node)
        if(parent_type %in% c("emph","strong")){
          style_settings <- c(style_settings, c(emph = "italic", strong = "bold")[[parent_type]])
        }else if(parent_type == "link"){
          link_path <- xml_attr(parent_node, "destination")
        }else if(parent_type == "image"){
          type <- "image"
          image_path <- xml_attr(parent_node, "destination")
        }
        parent_node <- xml2::xml_parent(parent_node)
      }

      run_content <- c(run_content,
                       apply_format_pptx_xml(
                         type,
                         xml_text(txt),
                         font_style = style_settings,
                         link_path = link_path,
                         image_path = image_path
                         ))

    }

    run_content <- paste0(run_content, collapse = "\n")

    glue('<a:p>{pPr}{run_content}</a:p>')
  },

  text = function(txt, font_style = NULL, link_path = NULL, ...) {
    a_rPr <- ""

    if (length(font_style) > 0 | length(link_path) > 0) {
      font_style_args <- style_xml_pptx(font_style)
      link_node <- ifelse(is.null(link_path),"", glue('<a:hlinkClick r:id=\"{link_path}\"/>'))
      a_rPr <-  glue("<a:rPr {font_style_args}>{link_node}</a:rPr>")
    }

    glue('<a:r>{a_rPr}<a:t>{txt}</a:t></a:r>')
  },


  image = function(txt, ..., image_path = NULL){
    stop("Images cannot be added via markdown formatting because images cannot be inline with text. Use as_file() instead.")
  }
)

#' Convert to xml nodes
#'
#' @param x an object to convert to xml nodes
#' @param ns word or pptx
#' @inheritParams rlang::args_dots_empty
#' @inheritParams rlang::args_error_context
#' @param .envir,.open,.close forwarded to [glue::glue()]
#'
#' @return an `xml_nodeset`
#'
#' @export
as_xml_nodeset <- function(x, ns = c("word", "pptx"), ..., .envir = parent.frame(), .open = "{", .close = "}", error_call = current_env()) {
  UseMethod("as_xml_nodeset")
}

#' @export
as_xml_nodeset.character <- function(x, ns = c("word", "pptx"), ..., .envir = parent.frame(), .open = "{", .close = "}", error_call = current_env()) {
  check_dots_empty(call = error_call)

  ns <- xml_nodeset_ns(ns, error_call = error_call)
  x <- glue_collapse(x)
  x <- glue("<polish-wrapper-node {ns}>{x}</polish-wrapper-node>")
  x <- tryCatch(
    glue(x, .envir = .envir, .open = .open, .close = .close),
    error = function(e) {
      x
    }
  )
  xml <- withCallingHandlers(
    suppressWarnings(read_xml(unclass(x))),
    error = function(err) {
      cli_abort(call = error_call, parent = err, c(
        "Invalid xml"
      ))
    }
  )
  children <- xml_children(xml)
  if (length(children) == 0L) {
    cli_abort(c(
      "{.arg x} must be an xml node."
    ), call = error_call)
  }
  add_class(children, "polish_xml_nodeset")
}

#' @export
as_xml_nodeset.glue <- function(x, ns = c("word", "pptx"), ..., .envir = parent.frame(), .open = "{", .close = "}", error_call = current_env()) {
  NextMethod()
}

#' @export
as_xml_nodeset.list <- function(x, ns = c("word", "pptx"), ..., .envir = parent.frame(), .open = "{", .close = "}", error_call = current_env()) {
  n <- length(x)
  if (n == 0) {
    cli_abort(c(
      "{.arg x} must have at least one element"
    ), call = error_call)
  }
  nodes <- map(x, as_xml_nodeset, ns = ns, ..., .envir = .envir, .open = .open, .close = .close, error_call = error_call)
  classes <- class(nodes[[1L]])
  if (n > 1L) {
    for (i in seq(2, n)) {
      nodes[[1]] <- append(nodes[[1]], nodes[[i]])
    }
  }
  class(nodes[[1]]) <- classes
  nodes[[1]]
}

#' xml namespaces
#'
#' @param ns word or pptx
#' @inheritParams rlang::args_error_context
#'
#' @return xml namespaces for word or pptx
#' @export
xml_nodeset_ns_spec <- function(ns = c("word", "pptx"), error_call = caller_env()) {
  ns <- arg_match(ns, error_call = error_call)
  switch(ns,
    word = c(
      'w'  = "http://schemas.openxmlformats.org/wordprocessingml/2006/main",
      'wp' = "http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing",
      'r'  = "http://schemas.openxmlformats.org/officeDocument/2006/relationships"
    ),
    pptx = c(
      'a' = "http://schemas.openxmlformats.org/drawingml/2006/main",
      'r' = "http://schemas.openxmlformats.org/officeDocument/2006/relationships",
      'p' = "http://schemas.openxmlformats.org/presentationml/2006/main",
      'w' = "http://schemas.openxmlformats.org/wordprocessingml/2006/main",
      'custom' = "urn:schemas-microsoft-com:office:custom-properties"
    )
  )
}

xml_nodeset_ns <- function(ns = c("word", "pptx"), error_call = caller_env()) {
  ns_spec <- xml_nodeset_ns_spec(ns = ns, error_call = error_call)
  glue_collapse(glue('xmlns:{names(ns_spec)}="{ns_spec}"'), sep = " ")
}

xml_name_pptx <- function(x) {
  xml_name(x, ns = xml_nodeset_ns_spec("pptx"))
}

xml_name_word <- function(x) {
  xml_name(x, ns = xml_nodeset_ns_spec("word"))
}

#' @export
as_xml_nodeset.xml_nodeset <- function(x, ns = c("word", "pptx"), ..., .envir = parent.frame(), .open = "{", .close = "}", error_call = current_env()) {
  check_dots_empty(call = error_call)
  add_class(x, "polish_xml_nodeset")
}

#' @export
as_xml_nodeset.xml_node <- function(x, ns = c("word", "pptx"), ..., .envir = parent.frame(), .open = "{", .close = "}", error_call = current_env()) {
  check_dots_empty(call = error_call)

  structure(list(x), class = c("polish_xml_nodeset", "xml_nodeset"))
}

#' @rdname as_xml_nodeset
#' @export
as_xml_node <- function(x, ..., .envir = parent.frame(), .open = "{", .close = "}", error_call = current_env()) {
  nodeset <- as_xml_nodeset(x, ..., .envir = .envir, .open = .open, .close = .close, error_call = error_call)
  if (length(nodeset) != 1L) {
    cli_abort(
      "{.arg x} must be a single xml node",
      call = error_call
    )
  }
  nodeset[[1L]]
}

#' @rdname as_xml_nodeset
#' @export
as_xml_node_word <- function(x, ..., .envir = parent.frame(), .open = "{", .close = "}", error_call = current_env()) {
  as_xml_node(x, ns = "word", ..., .envir = .envir, .open = .open, .close = .close, error_call = error_call)
}

#' @rdname as_xml_nodeset
#' @export
as_xml_node_pptx <- function(x, ..., .envir = parent.frame(), .open = "{", .close = "}", error_call = current_env()) {
  as_xml_node(x, ns = "word", ..., .envir = .envir, .open = .open, .close = .close, error_call = error_call)
}

#' @rdname as_xml_nodeset
#' @export
as_xml_word <- function(x, ..., .envir = parent.frame(), .open = "{", .close = "}", error_call = current_env()) {
  check_dots_empty(call = error_call)
  as_xml_nodeset(x, ns = "word", .envir = .envir, .open = .open, .close = .close, error_call = error_call)
}

#' @rdname as_xml_nodeset
#' @export
as_xml_pptx <- function(x, ..., .envir = parent.frame(), .open = "{", .close = "}", error_call = current_env()) {
  check_dots_empty(call = error_call)
  as_xml_nodeset(x, ns = "pptx", .envir = .envir, .open = .open, .close = .close, error_call = error_call)
}

#' @export
print.polish_xml_nodeset <- function(x, ...) {
  writeLines(as.character(x))
  invisible(x)
}

test_that("as_xml_nodeset() checks it's nodes", {
  expect_equal(xml_name(as_xml_nodeset("<hello />")), "hello")

  expect_snapshot(as_xml_nodeset("hello"), error = TRUE)
  expect_snapshot(as_xml_nodeset("<hello"), error = TRUE)
})

test_that("as_xml_nodeset(character())", {
  nodes <- as_xml_nodeset(c("<hello/>", "<world/>"))
  expect_s3_class(nodes, "polish_xml_nodeset")
  expect_equal(xml_name(nodes), c("hello", "world"))
})

test_that("as_xml_nodeset(list())", {
  content_string <- "<tbl>tblcontent</tbl>"
  content_nodeset <- as_xml_nodeset(content_string)

  nodes <- as_xml_nodeset(list("<hello/>", content_nodeset, "<world/>"))
  expect_s3_class(nodes, "polish_xml_nodeset")
  expect_equal(xml_name(nodes), c("hello", "tbl", "world"))

  nodes <- as_xml_nodeset(list("<hello/>", content_nodeset[[1L]], "<world/>"))
  expect_s3_class(nodes, "polish_xml_nodeset")
  expect_equal(xml_name(nodes), c("hello", "tbl", "world"))
})

test_that("as_xml_nodeset(character()) does glue", {
  who <- "world"
  nodes <- as_xml_nodeset('<hello who = "{who}"/>')
  expect_s3_class(nodes, "polish_xml_nodeset")
  expect_equal(xml_attr(nodes[[1]], "who"), "world")
})

test_that("as_xml_node(character()) does glue", {
  who <- "world"
  node <- as_xml_node('<hello who = "{who}"/>')
  expect_s3_class(node, "xml_node")
  expect_equal(xml_attr(node, "who"), "world")
})

test_that("as_xml_node_word(character()) does glue", {
  who <- "world"
  node <- as_xml_node_word('<hello who = "{who}"/>')
  expect_s3_class(node, "xml_node")
  expect_equal(xml_attr(node, "who"), "world")
})

test_that("as_xml_node_pptx(character()) does glue", {
  who <- "world"
  node <- as_xml_node_pptx('<hello who = "{who}"/>')
  expect_s3_class(node, "xml_node")
  expect_equal(xml_attr(node, "who"), "world")
})

test_that("polish_content_word(<flextable>)", {
  ft <- flextable::flextable(data.frame(x = 1:3, y = letters[1:3]))

  xml <- as_xml_node(polish_content_word(ft, inline = TRUE))
  expect_equal(xml_name_word(xml), "w:tbl")

  xml <- as_xml_node(polish_content_word(ft, inline = FALSE))
  expect_equal(xml_name_word(xml), "w:tbl")
})

test_that("polish_content_pptx(<flextable>)", {
  ft <- flextable::flextable(data.frame(x = 1:3, y = letters[1:3]))
  polished_ft <- polish_content_pptx(ft)

  expect_equal(xml_name_pptx(as_xml_node(polished_ft)), "p:graphicFrame")

  expect_snapshot(
    transform = transform_polish,
    polished_ft
  )

})

test_that("polish_content_pptx(<flextable>) keeps ph info", {
  ft <- flextable::flextable(data.frame(x = 1:3, y = letters[1:3]))

  ph <- '<p:nvSpPr><p:cNvPr id="c5672246-4915-4812-b4a2-e7de4805bade" name="New Placeholder"/><p:cNvSpPr><a:spLocks noGrp="1"/></p:cNvSpPr><p:nvPr><p:ph type="body"/></p:nvPr></p:nvSpPr><p:spPr><a:xfrm><a:off x="487680" y="480060"/><a:ext cx="7071360" cy="5829300"/></a:xfrm></p:spPr>'
  polished_ft <- polish_content_pptx(ft, ph = ph)

  expect_equal(xml_name_pptx(as_xml_node(polished_ft)), "p:graphicFrame")
  expect_equal(xml_attr(xml_find_all(polished_ft, ".//p:cNvPr"), "id"), "c5672246-4915-4812-b4a2-e7de4805bade")
  expect_equal(xml_attr(xml_find_all(polished_ft, ".//p:cNvPr"), "name"), "New Placeholder")
})

test_that("polish_content_pptx(<flextable>) applied table styling id", {

  ft <- flextable::flextable(data.frame(x = 1:3, y = letters[1:3]))

  ## define a basic table style in the pptx
  pptx <- list(rpptx = officer::read_pptx())
  cat(
    '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
      <a:tblStyleLst xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main">
      <a:tblStyle styleId="{291DBC07-340B-ACF5-F4C7-67FE120D59C4}" styleName="New Table Style"/>
      </a:tblStyleLst>
      ',
    file = file.path(pptx$rpptx$package_dir, "ppt/tableStyles.xml")
  )

  polished_ft <- polish_content_pptx(ft, table_style = list(style = "New Table Style"), pptx = pptx)

  expect_equal(xml_name_pptx(as_xml_node(polished_ft)), "p:graphicFrame")
  expect_equal(as_xml_node(polished_ft) |> xml_find_all(".//a:tblPr") |> as.character(),
               "<a:tblPr>\n  <a:tableStyleId>{291DBC07-340B-ACF5-F4C7-67FE120D59C4}</a:tableStyleId>\n</a:tblPr>")

  expect_snapshot(
    transform = transform_polish,
    polished_ft
  )

  ## test invalid style
  expect_error(
    polish_content_pptx(ft, table_style = list(style = "Invalid Style"), pptx = pptx),
    "Invalid Table Style Provided"
  )

})

test_that("polish_content_pptx(<flextable>) applied table styling option", {

  ft <- flextable::flextable(data.frame(x = 1:3, y = letters[1:3]))
  polished_ft <- polish_content_pptx(ft, table_style = list(options = "Header Row"))

  expect_equal(xml_name_pptx(as_xml_node(polished_ft)), "p:graphicFrame")
  expect_equal(as_xml_node(polished_ft) |> xml_find_all(".//a:tblPr") |> as.character(), "<a:tblPr firstRow=\"1\"/>")

  expect_snapshot(
    transform = transform_polish,
    polished_ft
  )

  ## invalid option does not prevent running still
  expect_snapshot(
    transform = transform_polish,
    polish_content_pptx(ft, table_style = list(options = "Invalid Option"))
  )

})

test_that("polish_content_pptx(<flextable>) for tables with empty rows", {

  ft <- flextable::flextable(data.frame(x = c(1:3,"",5), y = c(letters[1:3],"",letters[5]))) |>
    flextable::padding( padding.left = 1, padding.right = 1, padding.top = 0, padding.bottom = 0, part = "all")

  polished_ft <- polish_content_pptx(ft)

  expect_equal(xml_name_pptx(as_xml_node(polished_ft)), "p:graphicFrame")

  expect_snapshot(
    transform = transform_polish,
    polished_ft
  )

})

test_that("polish_content_word(<data.frame>)", {
  d_f <- data.frame(x = 1:3, y = letters[1:3])

  xml <- as_xml_node(polish_content_word(d_f))
  expect_equal(xml_name_word(xml), "w:tbl")
})

test_that("polish_content_pptx(<data.frame>)", {
  d_f <- data.frame(x = 1:3, y = letters[1:3])

  xml <- as_xml_node(polish_content_pptx(d_f))
  expect_equal(xml_name_pptx(xml), "p:graphicFrame")
})

#' Polish content for Powerpoint
#'
#' @param x object to be polished
#' @param ph placeholder
#' @param pptx The Powerpoint Presentation object
#' @inheritParams rlang::args_dots_empty
#' @inheritParams rlang::args_error_context
#'
#' @returns a "polish_xml_nodeset" of viable PowerPoint OOXML for addition
#'   to PowerPoint presentations.
#'
#' @export
polish_content_pptx <- function(x, ph = '<p:ph/>', pptx, ..., error_call = current_env()) {
  UseMethod("polish_content_pptx")
}

#' @export
polish_content_pptx.default <- function(x, ph = '<p:ph/>', pptx, ..., error_call = current_env()) {
  abort_default(x, error_call = error_call)
}

#' @export
polish_content_pptx.file <- function(x, ph = '<p:ph/>', pptx, ..., error_call = current_env()) {
  abort_unknown_extension(x, error_call = error_call)
}

#' @export
polish_content_pptx.as_is <- function(x, ph = '<p:ph/>', pptx, ..., error_call = current_env()) {
  not_yet_implemented("polish_content_pptx(<as_is>)", issue = 2, error_call = error_call)
}

#' @export
polish_content_pptx.as_md <- function(x, ph = '<p:ph/>', pptx, ..., error_call = current_env()) {

  ## convert md content to actual text
  content <- md_as_ppt_xml(x)

  ## Style each paragraph/content
  xml_content <- glue('
    <p:sp>
      {ph}
      <p:txBody>
        <a:bodyPr/>
        <a:lstStyle/>
        {content}
      </p:txBody>
    </p:sp>
  ')

  as_xml_pptx(xml_content, error_call = error_call)
}

#' @param escape Should `x` be escaped with [htmltools::htmlEscape()]
#' @param collapse If `TRUE` a `<a:br w:type="textWrapping" w:clear="all">` element is added between the `<a:r>` nodes
#' @param font_color color of text. Enter as a hex code.
#' @param font_style styling of text, vector including "bold","italic","underline".
#' @param font_size point size of text if different from document default.
#' @param font_typeface typeface of the text to be added if different from document default.
#'
#' @rdname polish_content_pptx
#' @export
polish_content_pptx.character <- function(x, ph = '<p:ph/>', pptx, ..., escape = TRUE, collapse = FALSE, font_color = NULL, font_style = NULL, font_size = NULL, font_typeface = NULL, error_call = current_env()){
  check_dots_empty(call = error_call)

  # polish each string into a <a:r> node
  # potentially separated by <a:br w:type="textWrapping" w:clear="all" />
  content <- glue_collapse(
    polish_pptx_strings(x, escape = escape, collapse = collapse, font_color = font_color, font_style = font_style, font_size = font_size, font_typeface = font_typeface, error_call = error_call)
  )

  xml_content <- glue('
    <p:sp>
      {ph}
      <p:txBody>
        <a:bodyPr/>
        <a:lstStyle/>
        <a:p>{content}</a:p>
      </p:txBody>
    </p:sp>
  ')
  as_xml_pptx(xml_content, error_call = error_call)
}

#' @export
polish_content_pptx.numeric <- function(x, ph = '<p:ph/>', pptx, ..., escape = TRUE, collapse = FALSE, error_call = current_env()) {
  polish_content_pptx(format(x, ...), ph = ph, pptx = pptx, escape = escape, collapse = collapse, ..., error_call = error_call)
}

#' @inheritParams polish_content_pptx.flextable
#' @export
polish_content_pptx.data.frame <- function(x, ph = '<p:ph/>', pptx, ..., table_style = NULL, error_call = current_env()) {
  check_dots_empty(call = error_call)
  ft <- flextable::flextable(x)
  polish_content_pptx(ft, ph = ph, pptx = pptx, ..., table_style = table_style, error_call = error_call)
}


#' @param list_type How to print the list. Options are "none", "unordered" and
#'   "ordered".
#'   - "none" (default) puts contents on new lines and indents lists accordingly.
#'   - "unordered" makes the content bulleted list with indents.
#'   - "ordered" makes a numbered list with intents.
#' @rdname polish_content_pptx
#' @export
polish_content_pptx.list <- function(x, ph = '<p:ph/>', pptx, ..., list_type = c("none","unordered","ordered"), error_call = current_env()){

  list_type <- match.arg(list_type)

  ## check all contents, even nested under another list is a character
  check_all_list_viable(x, error_call = error_call)


  ## convert md content to actual text
  content <- list_as_ppt_xml(x, ..., list_type = list_type, error_call = error_call) |>
    paste0(collapse = "\n")

  xml_content <- glue('
    <p:sp>
      {ph}
      <p:txBody>
        <a:bodyPr/>
        <a:lstStyle/>
        {content}
      </p:txBody>
    </p:sp>
  ')

  as_xml_pptx(xml_content, error_call = error_call)
}


# file_ -------------------------------------------------------------------

#' @param units see [officer::external_img]
#' @param image_fit Should the image be distorted to match the dimensions of the placeholder, or scaled up/down and keep dimension ratio (scale). Default is "stretch".
#' @param scale Multiplicative scaling factor to use when saving the plot. See [ggplot2::ggsave].
#'
#' @rdname polish_content_pptx
#' @export
polish_content_pptx.file_png <- function(x, ph = '<p:ph/>', pptx, ..., height = NULL, width = NULL, units = "in", image_fit = c("stretch","scale"), error_call = current_env()) {
  check_dots_empty(call = error_call)

  image_fit <- match.arg(image_fit)

  ## get image size, either pre-defined, or guess size
  if(is.null(height) & is.null(width)){
    ext_img <- officer::external_img(x, guess_size = TRUE)
  }else{
    ext_img <- officer::external_img(x, width = width, height = height, guess_size = FALSE)
  }

  image_definition <-  list(dims = attr(ext_img, "dims"), offset = list(top = 0, left = 0))

  # wrap ph in a <p:sp> node
  sp_ph      <- sp_shell(ph)

  ## If placeholder has dims, get them
  if (length(xml_find_all(sp_ph, ".//a:xfrm")) == 1) {
    ph_offsets <- dim_extract(sp_ph, ".//a:off", c("x", "y"))
    names(ph_offsets) <- c("left","top")
    ph_dims    <- dim_extract(sp_ph, ".//a:ext", c("cx", "cy"))
    names(ph_dims) <- c("width","height")

    if(image_fit == "scale"){
      image_definition <- image_fit_scale(image_definition = image_definition, ph_offsets = ph_offsets, ph_dims = ph_dims)
    }else{
      image_definition <- image_fit_stretch(image_definition = image_definition, ph_offsets = ph_offsets, ph_dims = ph_dims)
    }

  }

  # make the <p:pic> node
  xml <- as_xml_pptx(officer::to_pml(
    x      = ext_img,
    top    = image_definition$offset$top,
    left   = image_definition$offset$left,
    height = image_definition$dims$height,
    width  = image_definition$dims$width,
    add_ns  = TRUE,
    ln      = officer::sp_line(lwd = 0, linecmpd = "sng", lineend = "rnd"),
    ph      = as.character(xml_find_all(sp_ph, "//p:ph"))
  ))

  xml_replace(
    xml_find_all(xml, ".//p:nvPicPr/p:cNvPr"),  # created by to_pml()
    xml_find_all(sp_ph, ".//p:nvSpPr/p:cNvPr")  # given as ph=
  )

  xml
}

#' @export
polish_content_pptx.file_jpeg <- polish_content_pptx.file_png

#' @export
polish_content_pptx.file_svg <- polish_content_pptx.file_png

#' @export
polish_content_pptx.file_txt <- function(x, ph = '<p:ph/>', pptx, ..., error_call = current_env()){
  check_dots_empty(call = error_call)

  not_yet_implemented("polish_content_pptx(<file_text>)", issue = 1, error_call = error_call)
}

#' @export
polish_content_pptx.file_md <- function(x, ph = '<p:ph/>', pptx, ..., error_call = current_env()){
  check_dots_empty(call = error_call)

  polish_content_pptx(as_md(readLines(x)), ph = ph, error_call = error_call)
}

#' @export
polish_content_pptx.file_rtf <- function(x, ph = '<p:ph/>', pptx, ..., error_call = current_env()){
  check_dots_empty(call = error_call)

  png_path <- convert_rtf_file_to_png(x)
  polish_content_pptx(as_file(png_path), ph = ph, image_fit = "scale", error_call = error_call)
}

#' @export
polish_content_pptx.file_html <- function(x, ph = '<p:ph/>', pptx, ..., error_call = current_env()){
  check_dots_empty(call = error_call)

  polish_content_pptx(
    as_file(convert_html_file_to_png(x)),
    ph = ph, image_fit = "scale", error_call = error_call
  )

}

#' @export
polish_content_pptx.file_pdf <- function(x, ph = '<p:ph/>', pptx, ..., error_call = current_env()){
  check_dots_empty(call = error_call)

  polish_content_pptx(
    as_file(convert_pdf_file_to_png(x)),
    ph = ph, image_fit = "scale", error_call = error_call
  )
}


# utils -------------------------------------------------------------------

#' Wrap the placeholder into a <p:sp> shell
#'
#' @param ph placeholder
#' @inheritParams rlang::args_error_context
#'
#' @returns an object of class "polish_xml_nodeset". Contains XML that is viable
#'   PowerPoint OOXML for addition to PowerPoint documents.
#'
#'
#' @export
sp_shell <- function(ph, error_call = current_env()){
  xml_content <- glue('<p:sp>{ph}</p:sp>')
  nodes <- suppressWarnings(
    as_xml_pptx(xml_content, error_call = error_call)
  )

  ns <- c(
    'a' = "http://schemas.openxmlformats.org/drawingml/2006/main",
    'r' = "http://schemas.openxmlformats.org/officeDocument/2006/relationships",
    'p' = "http://schemas.openxmlformats.org/presentationml/2006/main"
  )
  names <- xml_children(nodes) |> xml_name(ns = ns)
  if (any(wrong <- !grepl("^p:", names))) {
    pos <- which(wrong)

    bullets <- glue("node {pos} is <{names[pos]}>.")
    names(bullets) <- rep("i", length(bullets))

    cli::cli_abort(c(
      "Nodes must be in the 'p:' namespace.",
      bullets
    ), call = error_call)
  }
  nodes
}

dim_extract <- function(xml, xpath, attrs = c("x, y")) {
  xml <- xml_find_first(xml, xpath)
  out <- purrr::map(attrs, \(name) {
    as.numeric(xml_attr(xml, name)) / 914400
  })
  names(out) <- attrs
  out
}

image_fit_scale <- function(image_definition, ph_offsets, ph_dims){

  ## get image ratio
  ratio <- image_definition$dims$height/image_definition$dims$width

  ##determine if scaling image for width or for height
  ### get potential new width/heights
  ratioed_height <- ratio * ph_dims$width
  ratioed_width <- ph_dims$height/ratio

  if(ratioed_height <= ph_dims$height){
    img_height <- ratioed_height
    img_width <- ph_dims$width
    img_top <- ph_offsets$top+((ph_dims$height-img_height)/2)
    img_left <- ph_offsets$left
  }else{
    img_height <- ph_dims$height
    img_width <- ratioed_width
    img_top <- ph_offsets$top
    img_left <- ph_offsets$left+((ph_dims$width-img_width)/2)
  }

  list(
    dims = list(height = img_height, width = img_width),
    offset = list(top = img_top, left = img_left)
  )

}

image_fit_stretch <- function(image_definition, ph_offsets, ph_dims){

  list(
    dims = ph_dims,
    offset = ph_offsets
  )

}

polish_pptx_strings <- function(txt, escape = TRUE, collapse = FALSE, font_color = NULL, font_style = NULL, font_size = NULL, font_typeface = NULL, error_call = caller_env()) {
  if (escape){
    txt <- htmlEscape(txt)
  }

  a_rPr <- style_text_pptx(font_color, font_style, font_size, font_typeface)

  xml <- glue('
      <a:r>{a_rPr}
      <a:t>{txt}</a:t>
    </a:r>
  ')

  xml <- glue_collapse(xml, sep = if(collapse) '<a:br w:type="textWrapping" w:clear="all"></a:br>' else "")
  as_xml_pptx(xml, error_call = error_call)
}

test_that("polish_content_pptx() fails on unknown class", {
  expect_snapshot(error = TRUE,
    polish_content_pptx(new_object("foo"))
  )
})

test_that("polish_content_pptx() fails on files with unknown extensions", {
  expect_snapshot(error = TRUE,
    polish_content_pptx(local_file(".foo"))
  )
})

test_that("polish_content_pptx().character", {
  txt <- "hello"
  xml <- polish_content_pptx(txt)
  expect_equal(xml_name(xml), "sp")
  expect_equal(xml_name(xml_children(xml)), c("ph", "txBody"))
  p_txBody <- xml_find_all(xml, "//p:sp/p:txBody")
  expect_equal(xml_name(xml_children(p_txBody)), c("bodyPr", "lstStyle", "p"))
  expect_equal(length(xml_find_all(xml, "//p:sp/p:txBody/a:p/a:r")), 1)
  expect_equal(xml_text(xml_find_all(xml, "//p:sp/p:txBody/a:p/a:r/a:t")), txt)

  txt <- c("hello", "world")
  xml <- polish_content_pptx(txt)
  expect_equal(xml_name(xml), "sp")
  expect_equal(xml_name(xml_children(xml)), c("ph", "txBody"))
  p_txBody <- xml_find_all(xml, "//p:sp/p:txBody")
  expect_equal(xml_name(xml_children(p_txBody)), c("bodyPr", "lstStyle", "p"))
  expect_equal(xml_name(xml_children(xml_find_all(xml, "//p:sp/p:txBody/a:p"))), c("r", "r"))
  expect_equal(length(xml_find_all(xml, "//p:sp/p:txBody/a:p/a:r")), 2)
  expect_equal(xml_text(xml_find_all(xml, "//p:sp/p:txBody/a:p/a:r/a:t")), txt)

  xml <- polish_content_pptx(txt, collapse = TRUE)
  expect_equal(xml_name(xml), "sp")
  expect_equal(xml_name(xml_children(xml)), c("ph", "txBody"))
  p_txBody <- xml_find_all(xml, "//p:sp/p:txBody")
  expect_equal(xml_name(xml_children(p_txBody)), c("bodyPr", "lstStyle", "p"))
  expect_equal(xml_name(xml_children(xml_find_all(xml, "//p:sp/p:txBody/a:p"))), c("r", "br", "r"))
  expect_equal(length(xml_find_all(xml, "//p:sp/p:txBody/a:p/a:r")), 2)
  expect_equal(xml_text(xml_find_all(xml, "//p:sp/p:txBody/a:p/a:r/a:t")), txt)
  br <- xml_find_all(xml, "//p:sp/p:txBody/a:p/a:br")
  expect_equal(xml_attr(br, "type"), "textWrapping")
  expect_equal(xml_attr(br, "clear"), "all")

  txt <- "hello"
  xml <- polish_content_pptx(txt, font_style = c("bold","italic"), font_color = "green")
  expect_equal(xml_name(xml), "sp")
  expect_equal(xml_name(xml_children(xml)), c("ph", "txBody"))
  p_txBody <- xml_find_all(xml, "//p:sp/p:txBody")
  expect_equal(xml_name(xml_children(p_txBody)), c("bodyPr", "lstStyle", "p"))
  expect_equal(length(xml_find_all(xml, "//p:sp/p:txBody/a:p/a:r")), 1)
  expect_equal(xml_text(xml_find_all(xml, "//p:sp/p:txBody/a:p/a:r/a:t")), txt)
  rPr <- xml_find_all(xml, "//p:sp/p:txBody/a:p/a:r/a:rPr")
  expect_equal(xml_attr(rPr, "b"), "1")
  expect_equal(xml_attr(rPr, "i"), "1")
  srgbClr <- xml_find_all(rPr, "a:solidFill/a:srgbClr")
  expect_equal(xml_attr(srgbClr, "val"), "00FF00")

  txt <- "hello"
  xml <- polish_content_pptx(txt, font_typeface = c("Helvetica"), font_size = 12)
  expect_equal(xml_name(xml), "sp")
  expect_equal(xml_name(xml_children(xml)), c("ph", "txBody"))
  p_txBody <- xml_find_all(xml, "//p:sp/p:txBody")
  expect_equal(xml_name(xml_children(p_txBody)), c("bodyPr", "lstStyle", "p"))
  expect_equal(length(xml_find_all(xml, "//p:sp/p:txBody/a:p/a:r")), 1)
  expect_equal(xml_text(xml_find_all(xml, "//p:sp/p:txBody/a:p/a:r/a:t")), txt)
  rPr <- xml_find_all(xml, "//p:sp/p:txBody/a:p/a:r/a:rPr")
  expect_equal(xml_attr(rPr, "sz"), "1200")
  expect_equal(xml_attr(xml_find_all(rPr, "a:latin"), "typeface"), "Helvetica")

  xml <- polish_content_pptx("<", escape = TRUE)
  expect_equal(xml_text(xml_find_all(xml, "//p:sp/p:txBody/a:p/a:r/a:t")), "<")

  expect_snapshot(error = TRUE,
    polish_content_pptx("<", escape = FALSE)
  )
})

test_that("polish_content_pptx.data.frame()", {

  expect_snapshot(transform = transform_polish,
    polish_content_pptx(data.frame(x = 1:2, y = c("hello", "world")))
  )

})

test_that("polish_content_pptx().numeric", {
  expect_snapshot(
    polish_content_pptx(42L)
  )
  expect_snapshot(
    polish_content_pptx(42.5)
  )

  expect_snapshot(
    polish_content_pptx(c(42L, 43L))
  )
  expect_snapshot(
    polish_content_pptx(c(42.5, 43.5))
  )
})

test_that("polish_content_word().ggplot", {
  suppressWarnings(withr::local_package("ggplot2"))

  expect_snapshot(transform = transform_polish,
    polish_content_pptx(ggplot(mtcars))
  )
})

test_that("polish_content_pptx.file_png() only creates one cNvPr", {
  suppressWarnings(withr::local_package("ggplot2"))

  ph <- "<p:nvSpPr><p:cNvPr id=\"c5672246-4915-4812-b4a2-e7de4805bade\" name=\"New Placeholder\"/><p:cNvSpPr><a:spLocks noGrp=\"1\"/></p:cNvSpPr><p:nvPr><p:ph type=\"body\"/></p:nvPr></p:nvSpPr><p:spPr><a:xfrm><a:off x=\"487680\" y=\"480060\"/><a:ext cx=\"7071360\" cy=\"5829300\"/></a:xfrm></p:spPr>"

  ## create png file
  p <- ggplot(mtcars, aes(mpg, cyl)) + geom_point()
  temp_png <- tempfile(fileext = ".png")
  ggsave(plot = p, filename = temp_png, width = 6, height = 5, units = "in", dpi = 300)
  img_file <- as_file(temp_png)

  xml <- polish_content_pptx(img_file, ph = ph)

  node_cNvPr <- xml2::xml_find_all(xml, ".//p:cNvPr")
  expect_equal(length(node_cNvPr), 1)
  expect_equal(xml2::xml_attr(node_cNvPr, "id"), "c5672246-4915-4812-b4a2-e7de4805bade")
  expect_equal(xml2::xml_attr(node_cNvPr, "name"), "New Placeholder")

  expect_equal(xml_attr(xml_find_all(xml, ".//a:off"), "x"), "487680")
  expect_equal(xml_attr(xml_find_all(xml, ".//a:off"), "y"), "480060")

  expect_equal(xml_attr(xml_find_all(xml, ".//a:ext"), "cx"), "7071360")
  expect_equal(xml_attr(xml_find_all(xml, ".//a:ext"), "cy"), "5829300")
})


test_that("polish_content_pptx.file_png() image_fit", {

  suppressWarnings(withr::local_package("ggplot2"))

  ph <- "<p:nvSpPr><p:cNvPr id=\"c5672246-4915-4812-b4a2-e7de4805bade\" name=\"New Placeholder\"/><p:cNvSpPr><a:spLocks noGrp=\"1\"/></p:cNvSpPr><p:nvPr><p:ph type=\"body\"/></p:nvPr></p:nvSpPr><p:spPr><a:xfrm><a:off x=\"487680\" y=\"480060\"/><a:ext cx=\"7071360\" cy=\"5829300\"/></a:xfrm></p:spPr>"

  ## create png file
  p <- ggplot(mtcars, aes(mpg, cyl)) + geom_point()
  temp_wide_png <- tempfile(fileext = ".png")
  ggsave(plot = p, filename = temp_wide_png, width = 6, height = 2, units = "in", dpi = 300)
  img_file_wide <- as_file(temp_wide_png)

  temp_tall_png <- tempfile(fileext = ".png")
  ggsave(plot = p, filename = temp_tall_png, width = 2, height = 6, units = "in", dpi = 300)
  img_file_tall <- as_file(temp_tall_png)

  ## scale image, keeping ratio
  xml_scale_wide <- polish_content_pptx(img_file_wide, ph = ph, image_fit = "scale")
  xml_scale_tall <- polish_content_pptx(img_file_tall, ph = ph, image_fit = "scale")

  ## wide, keep x offset and width, move y offset and height
  expect_equal(xml_attr(xml_find_all(xml_scale_wide, ".//a:off"), "x"), "487680")
  expect_equal(xml_attr(xml_find_all(xml_scale_wide, ".//a:off"), "y"), "2216150")
  expect_equal(xml_attr(xml_find_all(xml_scale_wide, ".//a:ext"), "cx"), "7071360")
  expect_equal(xml_attr(xml_find_all(xml_scale_wide, ".//a:ext"), "cy"), "2357120")

  ## tall, keep y offset and height, move x offset and width
  expect_equal(xml_attr(xml_find_all(xml_scale_tall, ".//a:off"), "x"), "3051810")
  expect_equal(xml_attr(xml_find_all(xml_scale_tall, ".//a:off"), "y"), "480060")
  expect_equal(xml_attr(xml_find_all(xml_scale_tall, ".//a:ext"), "cx"), "1943100")
  expect_equal(xml_attr(xml_find_all(xml_scale_tall, ".//a:ext"), "cy"), "5829300")

  ## stretch image to ph

  xml_stretch_wide <- polish_content_pptx(img_file_wide, ph = ph, image_fit = "stretch")
  xml_stretch_tall <- polish_content_pptx(img_file_tall, ph = ph, image_fit = "stretch")

  expect_equal(xml_attr(xml_find_all(xml_stretch_wide, ".//a:off"), "x"), "487680")
  expect_equal(xml_attr(xml_find_all(xml_stretch_wide, ".//a:off"), "y"), "480060")
  expect_equal(xml_attr(xml_find_all(xml_stretch_wide, ".//a:ext"), "cx"), "7071360")
  expect_equal(xml_attr(xml_find_all(xml_stretch_wide, ".//a:ext"), "cy"), "5829300")

  expect_equal(xml_attr(xml_find_all(xml_stretch_tall, ".//a:off"), "x"), "487680")
  expect_equal(xml_attr(xml_find_all(xml_stretch_tall, ".//a:off"), "y"), "480060")
  expect_equal(xml_attr(xml_find_all(xml_stretch_tall, ".//a:ext"), "cx"), "7071360")
  expect_equal(xml_attr(xml_find_all(xml_stretch_tall, ".//a:ext"), "cy"), "5829300")

})

expect_have_argument <- function(pattern, arg = "error_call") {
  ns <- asNamespace("polish")
  funs <- ls(ns, pattern = pattern)
  purrr::walk(funs, \(fun) {
    f <- get(fun, envir = ns)
    form <- formals(f)
    expect_true(arg %in% names(form), info = glue::glue("{fun}() must take an `{arg}=` argument"))
  })
}

# polish_content() fails with wrong type

    Code
      polish_content("", type = "xml")
    Condition <rlang_error>
      Error in `polish_content()`:
      ! `type` must be one of "word" or "pptx", not "xml".

# polish_content() fails on unknown class

    Code
      polish_content(foo, type = "word")
    Condition <polish_content_word_error>
      Error in `polish_content()`:
      ! Cannot polish content for word documents.
      Caused by error in `polish_content_word()`:
      ! No available method for objects of type <foo>.

---

    Code
      polish_content(foo, type = "pptx")
    Condition <polish_content_pptx_error>
      Error in `polish_content()`:
      ! Cannot polish content for pptx documents.
      Caused by error in `polish_content_pptx()`:
      ! No available method for objects of type <foo>.

# polish_content(type=) is mandatory

    Code
      polish_content(42)
    Condition
      Error in `polish_content()`:
      ! `type` must be one of "word" or "pptx", not absent.

# polish_content() fails on files with unknown extensions

    Code
      polish_content(file, type = "word")
    Condition <polish_content_word_error>
      Error in `polish_content()`:
      ! Cannot polish content for word documents.
      Caused by error in `polish_content_word()`:
      ! No available method for `.foo` files.

---

    Code
      polish_content(file, type = "pptx")
    Condition <polish_content_pptx_error>
      Error in `polish_content()`:
      ! Cannot polish content for pptx documents.
      Caused by error in `polish_content_pptx()`:
      ! No available method for `.foo` files.

# polish_content() checks that the method returns an xml_nodeset

    Code
      polish_content(foo, type = "word")
    Condition
      Error in `polish_content()`:
      ! Polished content must be an <xml_nodeset> object, not a string.
      i Check the `polish_content_word()` method objects of class <>.
        => polish_content_word.foo
        * polish_content_word.default

---

    Code
      polish_content(bar, type = "word")
    Condition
      Error in `polish_content()`:
      ! Polished content must be an <xml_nodeset> object, not a string.
      i Check the `polish_content_word()` method objects of class <>.
        => polish_content_word.bar
        -> polish_content_word.foo
        * polish_content_word.default

---

    Code
      polish_content(foo, type = "pptx")
    Condition
      Error in `polish_content()`:
      ! Polished content must be an <xml_nodeset> object, not a string.
      i Check the `polish_content_pptx()` method objects of class <>.
        => polish_content_pptx.foo
        * polish_content_pptx.default

---

    Code
      polish_content(bar, type = "pptx")
    Condition
      Error in `polish_content()`:
      ! Polished content must be an <xml_nodeset> object, not a string.
      i Check the `polish_content_pptx()` method objects of class <>.
        => polish_content_pptx.bar
        -> polish_content_pptx.foo
        * polish_content_pptx.default

# polish_content_word().ggplot - partially or incorrectly defined height and width,

    Code
      polish_content_pptx(p, ph = ph_dims_unset, height = NA, width = 12)
    Condition
      Error in `polish_content_pptx.ggplot()`:
      ! object 'width_value' not found

---

    Code
      polish_content_pptx(p, ph = ph_dims_unset, height = NULL, width = 12)
    Condition
      Error in `polish_content_pptx.ggplot()`:
      ! object 'width_value' not found

---

    Code
      polish_content_pptx(p, ph = ph_dims_unset, height = -10, width = 12)
    Condition
      Error in `polish_content_pptx.ggplot()`:
      ! object 'width_value' not found

---

    Code
      polish_content_pptx(p, ph = ph_dims_unset, height = c(10, 11), width = 12)
    Condition
      Error in `polish_content_pptx.ggplot()`:
      ! object 'width_value' not found

---

    Code
      polish_content_pptx(p, ph = ph_dims_unset, height = "invalid", width = 12)
    Condition
      Error in `polish_content_pptx.ggplot()`:
      ! object 'width_value' not found

# polish_content_word(<file_txt>)

    Code
      polish_content_word(as_file(tf), inline = TRUE)
    Output
      <w:r>
        <w:t>hello</w:t>
      </w:r>
      <w:br w:type="textWrapping" w:clear="all"/>
      <w:r>
        <w:t>world</w:t>
      </w:r>

---

    Code
      polish_content_word(as_file(tf), inline = FALSE)
    Output
      <w:p>
        <w:r>
          <w:t>hello</w:t>
        </w:r>
        <w:br w:type="textWrapping" w:clear="all"/>
        <w:r>
          <w:t>world</w:t>
        </w:r>
      </w:p>

# polish_content_word(<file_md>)

    Code
      polish_content_word(as_file(tf))
    Output
      <w:p>
        <w:pPr/>
        <w:r>
          <w:rPr/>
          <w:t xml:space="preserve">hello</w:t>
        </w:r>
        <w:br w:type="textWrapping" w:clear="right"/>
        <w:r>
          <w:rPr>
            <w:i/>
          </w:rPr>
          <w:t xml:space="preserve">world</w:t>
        </w:r>
      </w:p>

# polish_content_word(<file_docx>

    Code
      polish_content_word(as_file(tf))
    Output
      <w:altChunk xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" r:id="[...]"/>

# polish_content_word(<file_rtf>

    Code
      polish_content_word(as_file(tf))
    Output
      <w:altChunk r:id="[...]"/>

# polish_content_word(<file_html>)

    Code
      polish_content_word(as_file(tf))
    Output
      <w:altChunk r:id="[...]"/>

# polish_content_pptx(<file_txt>)

    Code
      polish_content_pptx(as_file(tf))
    Condition
      Error in `polish_content_pptx()`:
      ! polish_content_pptx(<file_text>) is not yet implemented.
      i See <https://github.com/GSK-Biostatistics/polish/issues/1> for progress.

# polish_content_pptx(<file_md>)

    Code
      polish_content_pptx(as_file(tf))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="0" indent="0">
              <a:buNone/>
            </a:pPr>
            <a:r>
              <a:t>hello</a:t>
            </a:r>
            <a:r>
              <a:rPr i="1"/>
              <a:t>world</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

# polish_content_pptx(<file_rtf>)

    Code
      polish_content_pptx(as_file(tf))
    Output
      <p:pic xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:p="http://schemas.openxmlformats.org/presentationml/2006/main">
        <p:nvPicPr>
          <p:cNvPr id="[...]"/>
          <p:cNvPicPr>
            <a:picLocks noGrp="1"/>
          </p:cNvPicPr>
          <p:nvPr>
            <p:ph/>
          </p:nvPr>
        </p:nvPicPr>
        <p:blipFill>
          <a:blip r:embed="[...]"/>
          <a:stretch>
            <a:fillRect/>
          </a:stretch>
        </p:blipFill>
        <p:spPr>
          <a:xfrm>
            <a:off x="[...]" y="[...]"/>
            <a:ext cx="[...]" cy="[...]"/>
          </a:xfrm>
          <a:prstGeom prst="rect">
            <a:avLst/>
          </a:prstGeom>
          <a:solidFill>
            <a:srgbClr val="FFFFFF">
              <a:alpha val="0"/>
            </a:srgbClr>
          </a:solidFill>
          <a:ln w="0" cap="rnd" cmpd="sng">
            <a:noFill/>
            <a:prstDash val="solid"/>
            <a:round/>
            <a:headEnd type="none" w="med" len="med"/>
            <a:tailEnd type="none" w="med" len="med"/>
          </a:ln>
        </p:spPr>
      </p:pic>

# polish_content_pptx(<file_html>)

    Code
      polish_content_pptx(as_file(tf))
    Output
      <p:pic xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:p="http://schemas.openxmlformats.org/presentationml/2006/main">
        <p:nvPicPr>
          <p:cNvPr id="[...]"/>
          <p:cNvPicPr>
            <a:picLocks noGrp="1"/>
          </p:cNvPicPr>
          <p:nvPr>
            <p:ph/>
          </p:nvPr>
        </p:nvPicPr>
        <p:blipFill>
          <a:blip r:embed="[...]"/>
          <a:stretch>
            <a:fillRect/>
          </a:stretch>
        </p:blipFill>
        <p:spPr>
          <a:xfrm>
            <a:off x="[...]" y="[...]"/>
            <a:ext cx="[...]" cy="[...]"/>
          </a:xfrm>
          <a:prstGeom prst="rect">
            <a:avLst/>
          </a:prstGeom>
          <a:solidFill>
            <a:srgbClr val="FFFFFF">
              <a:alpha val="0"/>
            </a:srgbClr>
          </a:solidFill>
          <a:ln w="0" cap="rnd" cmpd="sng">
            <a:noFill/>
            <a:prstDash val="solid"/>
            <a:round/>
            <a:headEnd type="none" w="med" len="med"/>
            <a:tailEnd type="none" w="med" len="med"/>
          </a:ln>
        </p:spPr>
      </p:pic>

# sp_shell()

    Code
      sp_shell("<p:ph/>")
    Output
      <p:sp>
        <p:ph/>
      </p:sp>

---

    Code
      sp_shell("<ph/>")
    Condition
      Error in `sp_shell()`:
      ! Nodes must be in the 'p:' namespace.
      i node 1 is <ph>.

---

    Code
      sp_shell("<ph/")
    Condition
      Error in `sp_shell()`:
      ! Invalid xml
      Caused by error in `read_xml.raw()`:
      ! error parsing attribute name [68]

# as_xml_nodeset() checks it's nodes

    Code
      as_xml_nodeset("hello")
    Condition
      Error in `as_xml_nodeset()`:
      ! `x` must be an xml node.

---

    Code
      as_xml_nodeset("<hello")
    Condition
      Error in `as_xml_nodeset()`:
      ! Invalid xml
      Caused by error in `read_xml.raw()`:
      ! error parsing attribute name [68]

# polish_content_pptx() fails on unknown class

    Code
      polish_content_pptx(new_object("foo"))
    Condition
      Error in `polish_content_pptx()`:
      ! No available method for objects of type <foo>.

# polish_content_pptx() fails on files with unknown extensions

    Code
      polish_content_pptx(local_file(".foo"))
    Condition
      Error in `polish_content_pptx()`:
      ! No available method for `.foo` files.

# polish_content_pptx().character

    Code
      polish_content_pptx("<", escape = FALSE)
    Condition
      Error in `polish_content_pptx()`:
      ! Invalid xml
      Caused by error in `read_xml.raw()`:
      ! StartTag: invalid element name [68]

# polish_content_pptx.data.frame()

    Code
      polish_content_pptx(data.frame(x = 1:2, y = c("hello", "world")))
    Output
      <p:graphicFrame xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:p="http://schemas.openxmlformats.org/presentationml/2006/main">
        <p:nvGraphicFramePr>
          <p:cNvPr id="[...]"/>
          <p:cNvGraphicFramePr>
            <a:graphicFrameLocks noGrp="true"/>
          </p:cNvGraphicFramePr>
          <p:nvPr/>
        </p:nvGraphicFramePr>
        <p:xfrm rot="0">
          <a:off x="[...]" y="[...]"/>
          <a:ext cx="[...]" cy="[...]"/>
        </p:xfrm>
        <a:graphic>
          <a:graphicData uri="http://schemas.openxmlformats.org/drawingml/2006/table">
            <a:tbl>
              <a:tblPr/>
              <a:tblGrid>
                <a:gridCol w="[...]"/>
                <a:gridCol w="[...]"/>
              </a:tblGrid>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>x</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>y</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>1</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>hello</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>2</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>world</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
            </a:tbl>
          </a:graphicData>
        </a:graphic>
      </p:graphicFrame>

# polish_content_pptx().numeric

    Code
      polish_content_pptx(42L)
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:r>
              <a:t>42</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(42.5)
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:r>
              <a:t>42.5</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(c(42L, 43L))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:r>
              <a:t>42</a:t>
            </a:r>
            <a:r>
              <a:t>43</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(c(42.5, 43.5))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:r>
              <a:t>42.5</a:t>
            </a:r>
            <a:r>
              <a:t>43.5</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

# polish_content_word().ggplot

    Code
      polish_content_pptx(ggplot(mtcars))
    Output
      <p:pic xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:p="http://schemas.openxmlformats.org/presentationml/2006/main">
        <p:nvPicPr>
          <p:cNvPr id="[...]"/>
          <p:cNvPicPr>
            <a:picLocks noGrp="1"/>
          </p:cNvPicPr>
          <p:nvPr>
            <p:ph/>
          </p:nvPr>
        </p:nvPicPr>
        <p:blipFill>
          <a:blip r:embed="[...]"/>
          <a:stretch>
            <a:fillRect/>
          </a:stretch>
        </p:blipFill>
        <p:spPr>
          <a:xfrm>
            <a:off x="[...]" y="[...]"/>
            <a:ext cx="[...]" cy="[...]"/>
          </a:xfrm>
          <a:prstGeom prst="rect">
            <a:avLst/>
          </a:prstGeom>
          <a:solidFill>
            <a:srgbClr val="FFFFFF">
              <a:alpha val="0"/>
            </a:srgbClr>
          </a:solidFill>
          <a:ln w="0" cap="rnd" cmpd="sng">
            <a:noFill/>
            <a:prstDash val="solid"/>
            <a:round/>
            <a:headEnd type="none" w="med" len="med"/>
            <a:tailEnd type="none" w="med" len="med"/>
          </a:ln>
        </p:spPr>
      </p:pic>

# polish_content_pptx(<flextable>)

    Code
      polished_ft
    Output
      <p:graphicFrame xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:p="http://schemas.openxmlformats.org/presentationml/2006/main">
        <p:nvGraphicFramePr>
          <p:cNvPr id="[...]"/>
          <p:cNvGraphicFramePr>
            <a:graphicFrameLocks noGrp="true"/>
          </p:cNvGraphicFramePr>
          <p:nvPr/>
        </p:nvGraphicFramePr>
        <p:xfrm rot="0">
          <a:off x="[...]" y="[...]"/>
          <a:ext cx="[...]" cy="[...]"/>
        </p:xfrm>
        <a:graphic>
          <a:graphicData uri="http://schemas.openxmlformats.org/drawingml/2006/table">
            <a:tbl>
              <a:tblPr/>
              <a:tblGrid>
                <a:gridCol w="[...]"/>
                <a:gridCol w="[...]"/>
              </a:tblGrid>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>x</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>y</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>1</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>a</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>2</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>b</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>3</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>c</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
            </a:tbl>
          </a:graphicData>
        </a:graphic>
      </p:graphicFrame>

# polish_content_pptx(<flextable>) applied table styling id

    Code
      polished_ft
    Output
      <p:graphicFrame xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:p="http://schemas.openxmlformats.org/presentationml/2006/main">
        <p:nvGraphicFramePr>
          <p:cNvPr id="[...]"/>
          <p:cNvGraphicFramePr>
            <a:graphicFrameLocks noGrp="true"/>
          </p:cNvGraphicFramePr>
          <p:nvPr/>
        </p:nvGraphicFramePr>
        <p:xfrm rot="0">
          <a:off x="[...]" y="[...]"/>
          <a:ext cx="[...]" cy="[...]"/>
        </p:xfrm>
        <a:graphic>
          <a:graphicData uri="http://schemas.openxmlformats.org/drawingml/2006/table">
            <a:tbl>
              <a:tblPr>
                <a:tableStyleId>{291DBC07-340B-ACF5-F4C7-67FE120D59C4}</a:tableStyleId>
              </a:tblPr>
              <a:tblGrid>
                <a:gridCol w="[...]"/>
                <a:gridCol w="[...]"/>
              </a:tblGrid>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>x</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>y</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>1</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>a</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>2</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>b</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>3</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>c</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                </a:tc>
              </a:tr>
            </a:tbl>
          </a:graphicData>
        </a:graphic>
      </p:graphicFrame>

# polish_content_pptx(<flextable>) applied table styling option

    Code
      polished_ft
    Output
      <p:graphicFrame xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:p="http://schemas.openxmlformats.org/presentationml/2006/main">
        <p:nvGraphicFramePr>
          <p:cNvPr id="[...]"/>
          <p:cNvGraphicFramePr>
            <a:graphicFrameLocks noGrp="true"/>
          </p:cNvGraphicFramePr>
          <p:nvPr/>
        </p:nvGraphicFramePr>
        <p:xfrm rot="0">
          <a:off x="[...]" y="[...]"/>
          <a:ext cx="[...]" cy="[...]"/>
        </p:xfrm>
        <a:graphic>
          <a:graphicData uri="http://schemas.openxmlformats.org/drawingml/2006/table">
            <a:tbl>
              <a:tblPr firstRow="1"/>
              <a:tblGrid>
                <a:gridCol w="[...]"/>
                <a:gridCol w="[...]"/>
              </a:tblGrid>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>x</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>y</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>1</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>a</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>2</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>b</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>3</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>c</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
            </a:tbl>
          </a:graphicData>
        </a:graphic>
      </p:graphicFrame>

---

    Code
      polish_content_pptx(ft, table_style = list(options = "Invalid Option"))
    Output
      <p:graphicFrame xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:p="http://schemas.openxmlformats.org/presentationml/2006/main">
        <p:nvGraphicFramePr>
          <p:cNvPr id="[...]"/>
          <p:cNvGraphicFramePr>
            <a:graphicFrameLocks noGrp="true"/>
          </p:cNvGraphicFramePr>
          <p:nvPr/>
        </p:nvGraphicFramePr>
        <p:xfrm rot="0">
          <a:off x="[...]" y="[...]"/>
          <a:ext cx="[...]" cy="[...]"/>
        </p:xfrm>
        <a:graphic>
          <a:graphicData uri="http://schemas.openxmlformats.org/drawingml/2006/table">
            <a:tbl>
              <a:tblPr/>
              <a:tblGrid>
                <a:gridCol w="[...]"/>
                <a:gridCol w="[...]"/>
              </a:tblGrid>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>x</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>y</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>1</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>a</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>2</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>b</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="r" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>3</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="63500" marR="63500">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="500"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="500"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>c</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="63500" marT="63500" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
            </a:tbl>
          </a:graphicData>
        </a:graphic>
      </p:graphicFrame>

# polish_content_pptx(<flextable>) for tables with empty rows

    Code
      polished_ft
    Output
      <p:graphicFrame xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:p="http://schemas.openxmlformats.org/presentationml/2006/main">
        <p:nvGraphicFramePr>
          <p:cNvPr id="[...]"/>
          <p:cNvGraphicFramePr>
            <a:graphicFrameLocks noGrp="true"/>
          </p:cNvGraphicFramePr>
          <p:nvPr/>
        </p:nvGraphicFramePr>
        <p:xfrm rot="0">
          <a:off x="[...]" y="[...]"/>
          <a:ext cx="[...]" cy="[...]"/>
        </p:xfrm>
        <a:graphic>
          <a:graphicData uri="http://schemas.openxmlformats.org/drawingml/2006/table">
            <a:tbl>
              <a:tblPr/>
              <a:tblGrid>
                <a:gridCol w="[...]"/>
                <a:gridCol w="[...]"/>
              </a:tblGrid>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="12700" marR="12700">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="0"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="0"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>x</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="0" marT="0" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="12700" marR="12700">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="0"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="0"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>y</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="0" marT="0" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="12700" marR="12700">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="0"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="0"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>1</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="0" marT="0" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="12700" marR="12700">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="0"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="0"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>a</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="0" marT="0" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="12700" marR="12700">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="0"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="0"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>2</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="0" marT="0" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="12700" marR="12700">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="0"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="0"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>b</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="0" marT="0" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="12700" marR="12700">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="0"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="0"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>3</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="0" marT="0" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="12700" marR="12700">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="0"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="0"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>c</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="0" marT="0" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="12700" marR="12700">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="0"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="0"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:endParaRPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                        <a:solidFill>
                          <a:srgbClr val="000000">
                            <a:alpha val="100000"/>
                          </a:srgbClr>
                        </a:solidFill>
                        <a:latin typeface="[...]"/>
                        <a:cs typeface="[...]"/>
                        <a:ea typeface="[...]"/>
                        <a:sym typeface="[...]"/>
                      </a:endParaRPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t/>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="0" marT="0" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="12700" marR="12700">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="0"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="0"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:endParaRPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                        <a:solidFill>
                          <a:srgbClr val="000000">
                            <a:alpha val="100000"/>
                          </a:srgbClr>
                        </a:solidFill>
                        <a:latin typeface="[...]"/>
                        <a:cs typeface="[...]"/>
                        <a:ea typeface="[...]"/>
                        <a:sym typeface="[...]"/>
                      </a:endParaRPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t/>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="0" marT="0" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
              <a:tr h="[...]">
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="12700" marR="12700">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="0"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="0"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>5</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="0" marT="0" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
                <a:tc>
                  <a:txBody>
                    <a:bodyPr/>
                    <a:lstStyle/>
                    <a:p>
                      <a:pPr algn="l" marL="12700" marR="12700">
                        <a:lnSpc>
                          <a:spcPct val="100000"/>
                        </a:lnSpc>
                        <a:spcBef>
                          <a:spcPts val="0"/>
                        </a:spcBef>
                        <a:spcAft>
                          <a:spcPts val="0"/>
                        </a:spcAft>
                        <a:buNone/>
                      </a:pPr>
                      <a:r>
                        <a:rPr cap="none" sz="1100" i="0" b="0" u="none" strike="noStrike">
                          <a:solidFill>
                            <a:srgbClr val="000000">
                              <a:alpha val="100000"/>
                            </a:srgbClr>
                          </a:solidFill>
                          <a:latin typeface="[...]"/>
                          <a:cs typeface="[...]"/>
                          <a:ea typeface="[...]"/>
                          <a:sym typeface="[...]"/>
                        </a:rPr>
                        <a:t>e</a:t>
                      </a:r>
                    </a:p>
                  </a:txBody>
                  <a:tcPr anchor="ctr" marB="0" marT="0" marR="0" marL="0">
                    <a:lnL algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnL>
                    <a:lnR algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnR>
                    <a:lnT algn="ctr" cmpd="sng" cap="flat" w="0">
                      <a:noFill/>
                      <a:prstDash val="solid"/>
                    </a:lnT>
                    <a:lnB algn="ctr" cmpd="sng" cap="flat" w="19050">
                      <a:solidFill>
                        <a:srgbClr val="666666">
                          <a:alpha val="100000"/>
                        </a:srgbClr>
                      </a:solidFill>
                      <a:prstDash val="solid"/>
                    </a:lnB>
                    <a:solidFill>
                      <a:srgbClr val="FFFFFF">
                        <a:alpha val="0"/>
                      </a:srgbClr>
                    </a:solidFill>
                  </a:tcPr>
                </a:tc>
              </a:tr>
            </a:tbl>
          </a:graphicData>
        </a:graphic>
      </p:graphicFrame>

# polish_content_word.as_md()

    Code
      polish_content_word(as_md("*hello*"))
    Output
      <w:p>
        <w:pPr/>
        <w:r>
          <w:rPr>
            <w:i/>
          </w:rPr>
          <w:t xml:space="preserve">hello</w:t>
        </w:r>
      </w:p>

# polish_content_pptx.as_md()

    Code
      polish_content_pptx(as_md("*hello*"))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="0" indent="0">
              <a:buNone/>
            </a:pPr>
            <a:r>
              <a:rPr i="1"/>
              <a:t>hello</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(as_md("__hello__"))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="0" indent="0">
              <a:buNone/>
            </a:pPr>
            <a:r>
              <a:rPr b="1"/>
              <a:t>hello</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(as_md("## hello"))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:r>
            <a:t>
              <text xml:space="preserve">hello</text>
            </a:t>
          </a:r>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(as_md("- hello"))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:r>
              <a:t>hello</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(as_md("- hello\n- world!"))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:r>
              <a:t>hello</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:r>
              <a:t>world!</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(as_md("- hello\n- *world*!"))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:r>
              <a:t>hello</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:r>
              <a:rPr i="1"/>
              <a:t>world</a:t>
            </a:r>
            <a:r>
              <a:t>!</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(as_md("- hello\n    - world!"))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:r>
              <a:t>hello</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr lvl="1"/>
            <a:r>
              <a:t>world!</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(as_md("- hello\n    - *world*!"))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:r>
              <a:t>hello</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr lvl="1"/>
            <a:r>
              <a:rPr i="1"/>
              <a:t>world</a:t>
            </a:r>
            <a:r>
              <a:t>!</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(as_md("1. hello"))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>hello</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(as_md("1. hello\n2. world!"))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>hello</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>world!</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(as_md("1. hello\n2. *world*!"))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>hello</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:rPr i="1"/>
              <a:t>world</a:t>
            </a:r>
            <a:r>
              <a:t>!</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(as_md("1. hello\n    1. world!"))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>hello</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="1028700" indent="-514350" lvl="1">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>world!</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(as_md("1. hello\n    1. *world*!"))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>hello</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="1028700" indent="-514350" lvl="1">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:rPr i="1"/>
              <a:t>world</a:t>
            </a:r>
            <a:r>
              <a:t>!</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx(as_md(
        "markdown is defined at [this website](https://daringfireball.net/projects/markdown/)"))
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="0" indent="0">
              <a:buNone/>
            </a:pPr>
            <a:r>
              <a:t>markdown is defined at </a:t>
            </a:r>
            <a:r>
              <a:rPr>
                <a:hlinkClick r:id="https://daringfireball.net/projects/markdown/"/>
              </a:rPr>
              <a:t>this website</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

# Expected Errors

    Code
      polish_content_pptx(as_md("![subtext](man/figures/logo.png)"))
    Condition
      Error in `format_pptx_xml[[node]]()`:
      ! Images cannot be added via markdown formatting because images cannot be inline with text. Use as_file() instead.

# as_is()

    Code
      polish_content_word(as_is("<a/>"))
    Condition
      Error in `polish_content_word()`:
      ! `x` must be a <w:p> or <w:tbl> node, not <a>.

---

    Code
      polish_content_word(as_is("<a"))
    Condition
      Error in `polish_content_word()`:
      ! Invalid xml
      Caused by error in `read_xml.raw()`:
      ! error parsing attribute name [68]

---

    Code
      polish_content_word(as_is("<a/><w:p/>"))
    Condition
      Error in `polish_content_word()`:
      ! `x` must be a <w:p> or <w:tbl> node, not <a>.

---

    Code
      polish_content_word(as_is("<a/><w:p/><b/>"))
    Condition
      Error in `polish_content_word()`:
      ! `x` must be a <w:p> or <w:tbl> node, not <a> or <b>.

# lists can be converted into bulleted lists - pptx

    Code
      polish_content_pptx.list(simple_list)
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="514350" indent="-514350"/>
            <a:r>
              <a:t>These</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350"/>
            <a:r>
              <a:t>are</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350"/>
            <a:r>
              <a:t>separate</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350"/>
            <a:r>
              <a:t>bullets</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx.list(simple_list, list_type = "ordered")
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>These</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>are</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>separate</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>bullets</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx.list(nested_list)
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="514350" indent="-514350"/>
            <a:r>
              <a:t>Non indented bullet 1</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="1028700" indent="-514350"/>
            <a:r>
              <a:t>indentet bullet 1</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="1028700" indent="-514350"/>
            <a:r>
              <a:t>indented bullet 2</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350"/>
            <a:r>
              <a:t>Non indented bullet 2</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

---

    Code
      polish_content_pptx.list(nested_list, list_type = "ordered")
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>Non indented bullet 1</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="1028700" indent="-514350" lvl="1">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>indentet bullet 1</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="1028700" indent="-514350" lvl="1">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>indented bullet 2</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="+mj-lt"/>
              <a:buAutoNum type="arabicPeriod"/>
            </a:pPr>
            <a:r>
              <a:t>Non indented bullet 2</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

# lists can be converted into indented text if content is text, numeric, or md - pptx

    Code
      polish_content_pptx.list(success_list, list_type = "none")
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="514350" indent="-514350"/>
            <a:r>
              <a:t>These</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350"/>
            <a:r>
              <a:t>1</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="0" indent="0">
              <a:buNone/>
            </a:pPr>
            <a:r>
              <a:rPr b="1"/>
              <a:t>hello</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

# lists can be converted into bulleted lists if content is text, numeric, or md - pptx

    Code
      polish_content_pptx.list(success_list, list_type = "unordered")
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="Arial" panose="020B0604020202020204" pitchFamily="34" charset="0"/>
              <a:buChar char="•"/>
            </a:pPr>
            <a:r>
              <a:t>These</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350">
              <a:buFont typeface="Arial" panose="020B0604020202020204" pitchFamily="34" charset="0"/>
              <a:buChar char="•"/>
            </a:pPr>
            <a:r>
              <a:t>1</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="0" indent="0">
              <a:buNone/>
            </a:pPr>
            <a:r>
              <a:rPr b="1"/>
              <a:t>hello</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

# lists can be converted into bulleted lists, apply styling - pptx

    Code
      polish_content_pptx.list(simple_list, font_color = "blue", font_style = "italic",
        font_size = 20)
    Output
      <p:sp>
        <p:ph/>
        <p:txBody>
          <a:bodyPr/>
          <a:lstStyle/>
          <a:p>
            <a:pPr marL="514350" indent="-514350"/>
            <a:r>
              <a:rPr sz="2000" i="1">
                <a:solidFill>
                  <a:srgbClr val="0000FF"/>
                </a:solidFill>
              </a:rPr>
              <a:t>These</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350"/>
            <a:r>
              <a:rPr sz="2000" i="1">
                <a:solidFill>
                  <a:srgbClr val="0000FF"/>
                </a:solidFill>
              </a:rPr>
              <a:t>are</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350"/>
            <a:r>
              <a:rPr sz="2000" i="1">
                <a:solidFill>
                  <a:srgbClr val="0000FF"/>
                </a:solidFill>
              </a:rPr>
              <a:t>separate</a:t>
            </a:r>
          </a:p>
          <a:p>
            <a:pPr marL="514350" indent="-514350"/>
            <a:r>
              <a:rPr sz="2000" i="1">
                <a:solidFill>
                  <a:srgbClr val="0000FF"/>
                </a:solidFill>
              </a:rPr>
              <a:t>bullets</a:t>
            </a:r>
          </a:p>
        </p:txBody>
      </p:sp>

# lists cannot be converted into bulleted lists if content is not text, numeric, or md - pptx

    Code
      polish_content_pptx.list(list(mtcars))
    Condition
      Error in `polish_content_pptx.list()`:
      ! List objects to be polished can only contain character or numeric values of length 1

# polish_content_word() fails on unknown class

    Code
      polish_content_word(new_object("foo"))
    Condition
      Error in `polish_content_word()`:
      ! No available method for objects of type <foo>.

# polish_content_word() fails on files with unknown extensions

    Code
      polish_content_word(local_file(".foo"))
    Condition
      Error in `polish_content_word()`:
      ! No available method for `.foo` files.

# polish_content_word.character()

    Code
      polish_content_word("<")
    Condition
      Error in `polish_content_word()`:
      ! Invalid xml
      Caused by error in `read_xml.raw()`:
      ! StartTag: invalid element name [68]

# polish_content_word.data.frame()

    Code
      polish_content_word(data.frame(x = 1:2, y = c("hello", "world")))
    Output
      <w:tbl xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main" xmlns:wp="http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships">
        <w:tblPr>
          <w:tblLayout w:type="autofit"/>
          <w:jc w:val="center"/>
          <w:tblW w:type="pct" w:w="5000"/>
          <w:tblLook w:firstRow="1" w:lastRow="0" w:firstColumn="0" w:lastColumn="0" w:noHBand="0" w:noVBand="1"/>
        </w:tblPr>
        <w:tr>
          <w:trPr>
            <w:tblHeader/>
          </w:trPr>
          <w:tc>
            <w:p>
              <w:pPr>
                <w:pStyle w:pstlname="Normal"/>
                <w:jc w:val="center"/>
              </w:pPr>
              <w:r>
                <w:t>x</w:t>
              </w:r>
            </w:p>
          </w:tc>
          <w:tc>
            <w:p>
              <w:pPr>
                <w:pStyle w:pstlname="Normal"/>
                <w:jc w:val="center"/>
              </w:pPr>
              <w:r>
                <w:t>y</w:t>
              </w:r>
            </w:p>
          </w:tc>
        </w:tr>
        <w:tr>
          <w:tc>
            <w:p>
              <w:pPr>
                <w:pStyle w:pstlname="Normal"/>
                <w:jc w:val="center"/>
              </w:pPr>
              <w:r>
                <w:t>1</w:t>
              </w:r>
            </w:p>
          </w:tc>
          <w:tc>
            <w:p>
              <w:pPr>
                <w:pStyle w:pstlname="Normal"/>
                <w:jc w:val="center"/>
              </w:pPr>
              <w:r>
                <w:t>hello</w:t>
              </w:r>
            </w:p>
          </w:tc>
        </w:tr>
        <w:tr>
          <w:tc>
            <w:p>
              <w:pPr>
                <w:pStyle w:pstlname="Normal"/>
                <w:jc w:val="center"/>
              </w:pPr>
              <w:r>
                <w:t>2</w:t>
              </w:r>
            </w:p>
          </w:tc>
          <w:tc>
            <w:p>
              <w:pPr>
                <w:pStyle w:pstlname="Normal"/>
                <w:jc w:val="center"/>
              </w:pPr>
              <w:r>
                <w:t>world</w:t>
              </w:r>
            </w:p>
          </w:tc>
        </w:tr>
      </w:tbl>

# polish_content_word().numeric

    Code
      polish_content_word(42L)
    Output
      <w:p>
        <w:r>
          <w:t xml:space="preserve">42</w:t>
        </w:r>
      </w:p>

---

    Code
      polish_content_word(42.5)
    Output
      <w:p>
        <w:r>
          <w:t xml:space="preserve">42.5</w:t>
        </w:r>
      </w:p>

---

    Code
      polish_content_word(c(42L, 43L))
    Output
      <w:p>
        <w:r>
          <w:t xml:space="preserve">42</w:t>
        </w:r>
      </w:p>
      <w:p>
        <w:r>
          <w:t xml:space="preserve">43</w:t>
        </w:r>
      </w:p>

---

    Code
      polish_content_word(c(42.5, 43.5))
    Output
      <w:p>
        <w:r>
          <w:t xml:space="preserve">42.5</w:t>
        </w:r>
      </w:p>
      <w:p>
        <w:r>
          <w:t xml:space="preserve">43.5</w:t>
        </w:r>
      </w:p>

# polish_content_word().ggplot

    Code
      polish_content_word(ggplot(mtcars))
    Output
      <w:p>
        <w:r xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main" xmlns:wp="http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:w14="http://schemas.microsoft.com/office/word/2010/wordml">
          <w:rPr/>
          <w:drawing xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships">
            <wp:inline distT="0" distB="0" distL="0" distR="0">
              <wp:extent cx="5486400" cy="4572000"/>
              <wp:docPr id="" name="" descr=""/>
              <wp:cNvGraphicFramePr>
                <a:graphicFrameLocks xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main" noChangeAspect="1"/>
              </wp:cNvGraphicFramePr>
              <a:graphic xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main">
                <a:graphicData uri="http://schemas.openxmlformats.org/drawingml/2006/picture">
                  <pic:pic xmlns:pic="http://schemas.openxmlformats.org/drawingml/2006/picture">
                    <pic:nvPicPr>
                      <pic:cNvPr id="" name=""/>
                      <pic:cNvPicPr>
                        <a:picLocks noChangeAspect="1" noChangeArrowheads="1"/>
                      </pic:cNvPicPr>
                    </pic:nvPicPr>
                    <pic:blipFill>
                      <a:blip r:embed="[...]"/>
                      <a:stretch>
                        <a:fillRect/>
                      </a:stretch>
                    </pic:blipFill>
                    <pic:spPr bwMode="auto">
                      <a:xfrm>
                        <a:off x="[...]" y="[...]"/>
                        <a:ext cx="[...]" cy="[...]"/>
                      </a:xfrm>
                      <a:prstGeom prst="rect">
                        <a:avLst/>
                      </a:prstGeom>
                      <a:noFill/>
                    </pic:spPr>
                  </pic:pic>
                </a:graphicData>
              </a:graphic>
            </wp:inline>
          </w:drawing>
        </w:r>
      </w:p>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xml.R
\name{xml_nodeset_ns_spec}
\alias{xml_nodeset_ns_spec}
\title{xml namespaces}
\usage{
xml_nodeset_ns_spec(ns = c("word", "pptx"), error_call = caller_env())
}
\arguments{
\item{ns}{word or pptx}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
xml namespaces for word or pptx
}
\description{
xml namespaces
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/show_xml.R
\name{show_xml}
\alias{show_xml}
\title{Show the xml of the polished version of a display in the browser}
\usage{
show_xml(xml, wrap = TRUE)
}
\arguments{
\item{xml}{an xml node or nodeset}

\item{wrap}{if TRUE, the xml is wrapped in a w:body node}
}
\value{
used for side effect of previewing xml in a browser. Helpful for
inspecting the OOXML generated by this package.
}
\description{
Show the xml of the polished version of a display in the browser
}
\details{
This opens the xml in the browser, so that you can do View source
and inspect the structure of the xml
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flextable.R, R/ggplot2.R, R/word.R
\name{polish_content_word.flextable}
\alias{polish_content_word.flextable}
\alias{polish_content_word.ggplot}
\alias{polish_content_word}
\alias{polish_content_word.character}
\alias{polish_content_word.file_png}
\title{Polish content for Word}
\usage{
\method{polish_content_word}{flextable}(
  x,
  inline = TRUE,
  ...,
  align = NULL,
  split = NULL,
  topcaption = TRUE,
  keepnext = NULL,
  error_call = current_env()
)

\method{polish_content_word}{ggplot}(
  x,
  inline = FALSE,
  ...,
  height = 5,
  width = 6,
  units = "in",
  res = 300,
  error_call = current_env()
)

polish_content_word(x, inline = FALSE, ..., error_call = current_env())

\method{polish_content_word}{character}(
  x,
  inline = FALSE,
  ...,
  font_color = NULL,
  font_style = NULL,
  font_size = NULL,
  font_typeface = NULL,
  error_call = current_env()
)

\method{polish_content_word}{file_png}(
  x,
  inline = FALSE,
  ...,
  height = 5,
  width = 6,
  units = "in",
  error_call = current_env()
)
}
\arguments{
\item{x}{object to be polished}

\item{inline}{if \code{FALSE} the comments are wrapped in a \verb{<w:p>}}

\item{...}{used in some methods}

\item{align}{alignment of table and caption}

\item{split}{should table be allowed to split}

\item{topcaption}{should caption be above or below the table}

\item{keepnext}{should content be marked for keep next.}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{width, height, units}{Plot size in \code{units} ("in", "cm", "mm", or "px").
defaults to 5 inches tall and 6 inches wide.}

\item{res}{Plot resolution. Also accepts a string input: "retina" (320),
"print" (300), or "screen" (72). Applies only to raster output types.}

\item{font_color}{color of text. Enter as a hex code.}

\item{font_style}{styling of text, vector including "bold","italic","underline".}

\item{font_size}{point size of text if different from document default.}

\item{font_typeface}{typeface of the text to be added if different from document default.}
}
\value{
an object of class "polish_xml_nodeset". Contains XML that is viable Word OOXML for addition to
Word documents.
}
\description{
Polish content for Word
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{polish_check_dots_empty}
\alias{polish_check_dots_empty}
\title{Check ... empty}
\usage{
polish_check_dots_empty(env = caller_env(), error = NULL, call = caller_env())
}
\arguments{
\item{env}{Environment in which to look for \code{...}.}

\item{error}{An optional error handler passed to \code{\link[rlang:try_fetch]{try_fetch()}}. Use
this e.g. to demote an error into a warning.}

\item{call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\description{
Wrapper around \code{\link[rlang:check_dots_empty]{rlang::check_dots_empty()}} that only does the check
if the \code{polish.dots_error} is set to TRUE
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_is.R
\name{as_is}
\alias{as_is}
\title{Mark as a node to inject as is}
\usage{
as_is(x, error_call = current_env())
}
\arguments{
\item{x}{raw ooxml passed through \code{\link[=as_xml_node]{as_xml_node()}}}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
adds the class "as_is" to the input text the content "as is" and not
modify it when polishing
}
\description{
Mark as a node to inject as is
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polish.R
\name{polish_content}
\alias{polish_content}
\title{Polish content}
\usage{
polish_content(x, type, ..., error_call = current_env(), error_class = NULL)
}
\arguments{
\item{x}{object to be polished into ooxml}

\item{type}{output type. Valid values are "word" and "pptx".}

\item{...}{additional arguments for methods}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{error_class}{Additional error class.}
}
\value{
an object of class "polish_xml_nodeset". Contains XML that is viable Word or PowerPoint OOXML for addition
to Word documents or PowerPoint presentations based on the "type".
}
\description{
Polish content
}
\details{
depending on \code{type} either the \code{\link[=polish_content_pptx]{polish_content_pptx()}} or \code{\link[=polish_content_word]{polish_content_word()}}
function is called.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xml.R
\name{as_xml_nodeset}
\alias{as_xml_nodeset}
\alias{as_xml_node}
\alias{as_xml_node_word}
\alias{as_xml_node_pptx}
\alias{as_xml_word}
\alias{as_xml_pptx}
\title{Convert to xml nodes}
\usage{
as_xml_nodeset(
  x,
  ns = c("word", "pptx"),
  ...,
  .envir = parent.frame(),
  .open = "{",
  .close = "}",
  error_call = current_env()
)

as_xml_node(
  x,
  ...,
  .envir = parent.frame(),
  .open = "{",
  .close = "}",
  error_call = current_env()
)

as_xml_node_word(
  x,
  ...,
  .envir = parent.frame(),
  .open = "{",
  .close = "}",
  error_call = current_env()
)

as_xml_node_pptx(
  x,
  ...,
  .envir = parent.frame(),
  .open = "{",
  .close = "}",
  error_call = current_env()
)

as_xml_word(
  x,
  ...,
  .envir = parent.frame(),
  .open = "{",
  .close = "}",
  error_call = current_env()
)

as_xml_pptx(
  x,
  ...,
  .envir = parent.frame(),
  .open = "{",
  .close = "}",
  error_call = current_env()
)
}
\arguments{
\item{x}{an object to convert to xml nodes}

\item{ns}{word or pptx}

\item{...}{These dots are for future extensions and must be empty.}

\item{.envir, .open, .close}{forwarded to \code{\link[glue:glue]{glue::glue()}}}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
an \code{xml_nodeset}
}
\description{
Convert to xml nodes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pptx.R
\name{sp_shell}
\alias{sp_shell}
\title{Wrap the placeholder into a <p:sp> shell}
\usage{
sp_shell(ph, error_call = current_env())
}
\arguments{
\item{ph}{placeholder}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
an object of class "polish_xml_nodeset". Contains XML that is viable
PowerPoint OOXML for addition to PowerPoint documents.
}
\description{
Wrap the placeholder into a <p:sp> shell
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/file.R
\name{as_file}
\alias{as_file}
\title{Mark as a file by adding classes}
\usage{
as_file(path)
}
\arguments{
\item{path}{file path}
}
\value{
\code{path} with classes set to \code{"file"} and \code{"file_{ext}"} where \code{ext} is
the file extension
}
\description{
Mark as a file by adding classes
}
\examples{
tf <- tempfile(fileext = ".txt")
writeLines("", tf)

as_file(tf)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flextable.R, R/ggplot2.R, R/pptx.R
\name{polish_content_pptx.flextable}
\alias{polish_content_pptx.flextable}
\alias{polish_content_pptx.ggplot}
\alias{polish_content_pptx}
\alias{polish_content_pptx.character}
\alias{polish_content_pptx.list}
\alias{polish_content_pptx.file_png}
\title{Polish content for Powerpoint}
\usage{
\method{polish_content_pptx}{flextable}(
  x,
  ph = "<p:ph/>",
  ...,
  full_width = FALSE,
  page_size = NULL,
  table_style = NULL,
  pptx = NULL,
  error_call = current_env()
)

\method{polish_content_pptx}{ggplot}(
  x,
  ph = "<p:ph/>",
  pptx,
  ...,
  height = NULL,
  width = NULL,
  res = 300,
  device = c("png", "svg", "jpeg"),
  scale = 1,
  error_call = current_env()
)

polish_content_pptx(x, ph = "<p:ph/>", pptx, ..., error_call = current_env())

\method{polish_content_pptx}{character}(
  x,
  ph = "<p:ph/>",
  pptx,
  ...,
  escape = TRUE,
  collapse = FALSE,
  font_color = NULL,
  font_style = NULL,
  font_size = NULL,
  font_typeface = NULL,
  error_call = current_env()
)

\method{polish_content_pptx}{list}(
  x,
  ph = "<p:ph/>",
  pptx,
  ...,
  list_type = c("none", "unordered", "ordered"),
  error_call = current_env()
)

\method{polish_content_pptx}{file_png}(
  x,
  ph = "<p:ph/>",
  pptx,
  ...,
  height = NULL,
  width = NULL,
  units = "in",
  image_fit = c("stretch", "scale"),
  error_call = current_env()
)
}
\arguments{
\item{x}{object to be polished}

\item{ph}{placeholder}

\item{...}{These dots are for future extensions and must be empty.}

\item{full_width}{Logical, used to fit full size of given width}

\item{page_size}{Number of rows to accommodate in a slide}

\item{table_style}{Named list with two entries: style and options. style is
the name of table style to apply as a character, options a character vector
of options to apply from the table style, including: "Banded Columns",
"Banded Rows", "First Column", "Header Row", "Last Column", and "Total Row"}

\item{pptx}{The Powerpoint Presentation object}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{width, height}{defaults to match the height and width of the placeholder
provided. When that cannot be derived, it assumes the plot should be 5
inches tall and 6 inches wide. Users may set the width and height, but they
most both be set.}

\item{res}{Plot resolution. Also accepts a string input: "retina" (320),
"print" (300), or "screen" (72). Applies only to raster output types.}

\item{device}{what graphic device to use when saving the plot. Can be one of
"png","svg", or "jpeg".}

\item{scale}{Multiplicative scaling factor to use when saving the plot. See \link[ggplot2:ggsave]{ggplot2::ggsave}.}

\item{escape}{Should \code{x} be escaped with \code{\link[htmltools:htmlEscape]{htmltools::htmlEscape()}}}

\item{collapse}{If \code{TRUE} a \verb{<a:br w:type="textWrapping" w:clear="all">} element is added between the \verb{<a:r>} nodes}

\item{font_color}{color of text. Enter as a hex code.}

\item{font_style}{styling of text, vector including "bold","italic","underline".}

\item{font_size}{point size of text if different from document default.}

\item{font_typeface}{typeface of the text to be added if different from document default.}

\item{list_type}{How to print the list. Options are "none", "unordered" and
"ordered".
\itemize{
\item "none" (default) puts contents on new lines and indents lists accordingly.
\item "unordered" makes the content bulleted list with indents.
\item "ordered" makes a numbered list with intents.
}}

\item{units}{see \link[officer:external_img]{officer::external_img}}

\item{image_fit}{Should the image be distorted to match the dimensions of the placeholder, or scaled up/down and keep dimension ratio (scale). Default is "stretch".}
}
\value{
a "polish_xml_nodeset" of viable PowerPoint OOXML for addition
to PowerPoint presentations.
}
\description{
Polish content for Powerpoint
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_is.R
\name{as_md}
\alias{as_md}
\title{Mark text as markdown}
\usage{
as_md(x)
}
\arguments{
\item{x}{text to be interpreted as markdown}
}
\value{
adds the class "as_ms" to the input text the content "as markdown"
apply markdown formatting to the text when polishing
}
\description{
Mark text as markdown
}
